
C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE IMG_FFT_FILL_ROB     *****       *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED  7-AUG-1990 BY T. BAKER.                        *
C  ********************************************************************
C
      SUBROUTINE IMG_FFT_FILL_ROB (IMG, NCOL, NROW, FFT,IDIM1,IDIM2,
     1           imgBkgP)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J
      integer*4 img_bkg,imgBkgP
      INTEGER*2 IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), TMP, fft_fzero
      LOGICAL   fft_full
C  -----------------------------------------------------------------------------
C
      img_bkg = imgBkgP
      FFT_FZERO = 0.0
C
      FFT_FULL = .TRUE.
      CALL FFT_CLEAR (FFT, IDIM1, IDIM2, fft_full)      ! ZERO THE FFT ARRAY
C
      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = FLOAT(IMG(I,J) - IMG_BKG)
          FFT((2*I)-1,J) = TMP
          FFT_FZERO      = FFT_FZERO + TMP
        ENDDO
      ENDDO
C
      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *                *****     SUBROUTINE FFT_2D     *****                *
C  *                                                                     *
C  *    COMPUTE IN-CORE 2D FFT OF THE DATA LOADED INTO THE FULL-SIZE     *
C  *     "FFT" ARRAY.                                                    *
C  *                                                                     *
C  *    ARGUMENTS: ARRAY, NN, NN_LENGTH, KSIGN, IFORM, WORK.             *
C  *    ARRAY     = REAL OR COMPLEX ARRAY TO BE TRANSFORMED.             *
C  *    NN        = ARRAY CONTAINING THE TRANSFORM DIMENSIONS.           *
C  *    NN_LENGTH = LENGTH OF NN ARRAY (I.E. 2 FOR 2D ARRAY).            *
C  *    KSIGN     = +1 FOR FORWARD TRANSFORM.                            *
C  *              = -1 FOR REVERSE TRANSFORM.                            *
C  *    IFORM     =  0 FOR ALL REAL INPUT DATA.                          *
C  *              =  1 FOR ALL OTHER TYPES.                              *
C  *                                                                     *
C  *                                                                     *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED 15-JAN-1992 BY T. BAKER.                           *
C  ***********************************************************************
C
      SUBROUTINE fft_2d_rob (fft, IDIM1, IDIM2, ksign, iform)
C
      IMPLICIT NONE
C
      INTEGER*4 NN(2), IDIM1, IDIM2, I, J, ksign, iform
      REAL*4    FFT(2*IDIM1,IDIM2), WORK, div
C  ---------------------------------------------------------------------------
C
Ca  ORIG way:
Ca      NN(1) = IDIM1
Ca      NN(2) = IDIM2
Ca
Ca      CALL FOURT (FFT, NN, 2, ksign, iform, WORK)    ! TRANSFORM THE DATA
Ca
      NN(1) = IDIM1
      NN(2) = IDIM2
      div   = FLOAT(IDIM1)*FLOAT(IDIM2)
C
      CALL FOURT (FFT, NN, 2, ksign, iform, WORK)     ! TRANSFORM THE DATA

      if(ksign .eq. -1) then
         DO J = 1,IDIM2
            DO I = 1,2*IDIM1
             FFT(I,J) = FFT(I,J)/div
            ENDDO
         ENDDO
      endif
C
      RETURN
      END
C  ---------------------------------------------------------------------------
C
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FFT_HIPLOP     *****               *
C  *                                                                          *
C  *    APPLY HI/LOW-PASS FILTER TO FFT DATA                                  *
C  *                                                                          *
C  *    RES_MAX = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS BELOW WHICH FFT  *
C  *              IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         *
C  *    RES_MIN = RESOLUTION IN # OF TRANSFORM SAMPLE POINTS ABOVE WHICH FFT  *
C  *              IS GIVEN FULL WEIGHT (WEIGHT = 1.0)                         *
C  *    CUT_MAX & CUT_MIN not yet implemented                                 *
C  *                                                                          *
C  *    This routine is based on the routines fft_hip() and fft_lop() in      *
C  *    fftlib.f                                                              *
C  *                                                                          *
C  *    May 2000  PURDUE UNIVERSITY, RWA.                                     *
C  ****************************************************************************
C
      SUBROUTINE FFT_HIPLOP (FFT, IDIM1, IDIM2, RES_Min, RES_Max,
     1       CUT_Min, CUT_Max, INVERTSWITCH, FFT_FULL)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, IX, IY, IY2, ICOUNT, XTEST, YTEST
      REAL*4    FFT(1), RES_MAX, CUT_MAX, DELSQmax, MAX_SQ, YSQ, RSQ
      REAL*4    RES_Min, CUT_Min, DELSQmin, Min_SQ
      REAL*4    WEIGHT
      LOGICAL   FFT_FULL, invertSwitch, weightIt
C  ----------------------------------------------------------------------------
C
      MAX_SQ = RES_MAX*RES_MAX
      DELSQmax  = CUT_MAX**2
      Min_SQ = RES_Min*RES_Min
      DELSQmin  = CUT_Min**2
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      ICOUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           YSQ = FLOAT(IY-IDIM2-1)**2
         ELSE
           YSQ = FLOAT(IY-1)**2
        ENDIF
        DO IX = 1,IDIM1                  ! PROCESS CURRENT FFT ROW
          ICOUNT = ICOUNT + 2
          IF(IX.LE.XTEST) THEN
             RSQ = YSQ + FLOAT(IX-1)**2            ! RIGHT HALF OF FFT
           ELSE
             RSQ = YSQ + FLOAT(IX-IDIM1-1)**2            ! LEFT HALF OF FFT
          ENDIF

Ca       Now let's test everything 
          if(.not.invertSwitch) then  ! Nornmal
            if(RSQ.gt.MAX_SQ .or.rsq.lt.min_sq) then      ! WEIGHT < 1.0
              if(DELSQmax.EQ.0.0 .and. DELSQmin.eq.0.0) then
                 WEIGHT = 0.0                        ! SHARP CUTOFF
               else 
Ca   Need to add code to do sloping cutoff
Ca                 if( (RES_MAX .gt. 0.0).and. 
Ca      1         (((sqrt(RSQ)-RES_MAX)**2/DELSQ).lt.50.0)) then
Ca                    WEIGHT = EXP(-((SQRT(RSQ)-RES_MAX)**2/DELSQ))
Ca                  else 
Ca                    if( RSQ/DELSQ .lt. 50.0) then
Ca                    WEIGHT = EXP(-RSQ/DELSQ)
Ca                  else
Ca                    WEIGHT = 0.0
Ca                  end if
Ca                 endif
                weight = 0.0  ! For now ...
              endif
              FFT(ICOUNT)   = WEIGHT*FFT(ICOUNT)
              FFT(ICOUNT+1) = WEIGHT*FFT(ICOUNT+1)
            ENDIF
          else   ! else it is invertSwitch 
            if(RSQ.lt.max_sq .and.rsq.gt.min_sq) then      ! WEIGHT < 1.0
              if(DELSQmax.EQ.0.0 .and. DELSQmin.eq.0.0) then
                 WEIGHT = 0.0                        ! SHARP CUTOFF
               else 
Ca   Need to add code to do sloping cutoff
Ca                 if( (RES_MAX .gt. 0.0).and. 
Ca      1         (((sqrt(RSQ)-RES_MAX)**2/DELSQ).lt.50.0)) then
Ca                    WEIGHT = EXP(-((SQRT(RSQ)-RES_MAX)**2/DELSQ))
Ca                  else 
Ca                    if( RSQ/DELSQ .lt. 50.0) then
Ca                    WEIGHT = EXP(-RSQ/DELSQ)
Ca                  else
Ca                    WEIGHT = 0.0
Ca                  end if
Ca                 endif
                weight = 0.0  ! For now ...
              endif
              FFT(ICOUNT)   = WEIGHT*FFT(ICOUNT)
              FFT(ICOUNT+1) = WEIGHT*FFT(ICOUNT+1)
            ENDIF
          endif  ! If .not. invertSwitch
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE CTF_MULTIPLY_rob   ***               *
C  *                                                                          *
C  *    MULTIPLY IMAGE FFT BY CTF.                                            *
C  *                                                                          *
C  *    PHI     = ANGLE (IN RADIANS) SPECIFYING THE DIRECTION OF FOCUS_MAJ    *
C  *               RELATIVE TO THE X-AXIS OF THE DIGITIZED IMAGE.             *
C  *    PSI     = ANGLE (IN RADIANS) OF A PARTICULAR FOURIER COMPONENT WHOSE  *
C  *               DEFOCUS VALUE IS TO BE CALCULATED.                         *
C  *    TEMPFAC = TEMPERATURE FACTOR, SPECIFIED IN ANGSTROMS SQUARED, IS      *
C  *               USED TO DAMPEN HIGH RESOUTION FEATURES.                    *
C  *                                                                          *
C  *    CTFMODE = 0 DO NOTHING!                                               *
C  *            = 1 MODIFY PHASES AND AMPLITUDES                              *
C  *            = 2 FLIP PHASES WHERE NECESSARY; LEAVE AMPLITUDES UNCHANGED   *
C  *            = 3 ADJUST AMPLITUDES; LEAVE PHASES UNCHANGED                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 13-AUG-1999 BY TSB (Fixed XTEST,YTEST problems).             *
C  *    MODIFIED  2-MAY-2000 BY TSB (Added CTF_MODE switch).                  *
C  *    LAST MODIFIED  2-MAY-2000 BY TSB/RWA (Implement CTF decay).           *
C  ****************************************************************************
C
      SUBROUTINE CTF_MULTIPLY_rob (CTFMODE, FFT, IDIM1, IDIM2, UNITS, 
     1 PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, 
     2 AMP_FAC, Cs, Cc, BETA, dE, VOLTS, TEMPFAC, FFT_FULL,
     3 filter, filt_fac)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, COUNT, IX, IY, IY2, XTEST, YTEST
      INTEGER*4 UNITS, CTFMODE, filter(10), filtTest
      REAL*4    FFT(1), YSQ, C, CTF, D_STAR, PIXSIZE, TMP
      REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, PSI, PHI, X, Y
      REAL*4    PI, RADIAN, WAVELENGTH, dF, AMP_FAC, Cs
      REAL*4    TEMPFAC, T, TWOPI, FACTOR, Cc, BETA, dE, VOLTS
      real*4    filt_fac(5), fft_rad_test, fft_rad
      LOGICAL   FFT_FULL, ASTIG
C  ----------------------------------------------------------------------------
C
      IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) RETURN
C
      TWOPI = 2.0*PI
      PHI   = ANG_MAJ * RADIAN        ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
      IF(UNITS.EQ.1) THEN
         TMP = 10.0/(IDIM1*PIXSIZE)                   ! CONVERT TO NM UNITS
       ELSE
         TMP = 1.0/(IDIM1*PIXSIZE)            ! ALREADY IN NM OR PIXEL UNITS
      ENDIF
C
      IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
         ASTIG = .FALSE.
         dF    = FOCUS_MAJ
       ELSE
         ASTIG = .TRUE.
      ENDIF
C
      XTEST  = IDIM1/2
      YTEST  = IDIM2/2
      COUNT = -1
C
      IF(FFT_FULL) THEN
         IY2 = IDIM2
       ELSE
         IY2 = YTEST + 1
      ENDIF
C
      DO IY = 1,IY2                        ! PROCESS FFT A ROW AT A TIME
        IF(IY.GT.YTEST) THEN
           Y = FLOAT(IY-IDIM2-1)
         ELSE
           Y = FLOAT(IY-1)
        ENDIF
        YSQ = Y**2
        DO IX = 1,IDIM1                  ! PROCESS CURRENT ROW
          COUNT = COUNT + 2
          IF(IX.LE.XTEST) THEN
             X = FLOAT(IX-1)                  ! RIGHT HALF OF FFT
           ELSE
             X = FLOAT(IX-IDIM1-1)            ! LEFT HALF OF FFT
          ENDIF
          D_STAR = SQRT(YSQ + X**2) * TMP
          IF(ABS(X).GT.0.9) THEN            ! TRAP FOR FFT Y-AXIS DATA
             PSI = ATAN2 (Y,X)            ! ATAN2 GIVES ANGLE IN RADIANS
           ELSE
             IF(Y.GE.0.0) THEN
                PSI =  PI/2.0
              ELSE
                PSI = -PI/2.0
             ENDIF
          ENDIF
          IF(ASTIG) CALL CTF_ASTIG (dF,FOCUS_MAJ,FOCUS_MIN,PSI,PHI)
          C = CTF (D_STAR, PI, WAVELENGTH, dF, AMP_FAC, Cs, Cc, BETA, 
     1             dE, VOLTS)
          T = EXP( -((TEMPFAC/400.0) * D_STAR**2) )            ! ADJUSTED FOR NM


Ca          print *, T, tfac, d_star
C           ****************************
C-----------*  APPLY CORRECT FILTER    *
C           ****************************
          factor = 1.0
          filtTest = 0
          IF(FILTER(5) .eq. 1) THEN
             filtTest = 1
             CALL CTF_FIRSTPEAK_WEI (dF, WAVELENGTH, FFT_RAD, 
     1          PIXSIZE, IDIM1,Cs, PI, units, amp_fac)
             FFT_RAD_TEST = D_STAR / TMP
             IF (FFT_RAD_TEST .LE. FFT_RAD) THEN
              FACTOR = factor * (T/(ABS(C) + FILT_FAC(1)*(1.0-ABS(C))))
             ELSE
              FACTOR = factor * ((T*abs(C))/(C*C    + 
     1                         FILT_FAC(2)*(1.0-abs(C))))
             ENDIF
C
          endif
          IF(FILTER(4) .eq. 1) THEN
             filtTest = 1
             FACTOR = factor * ((T*ABS(C))/(C*C + 
     1                filt_fac(1)*(1.0-ABS(C))))  ! "Misha+Wei"
          endif 
          IF(FILTER(3) .eq. 1) THEN
             filtTest = 1
             FACTOR = factor * ((T*ABS(C))/(C*C + 
     1                         filt_fac(1)))      ! "Misha" filter
          endif
          IF(FILTER(2) .eq. 1) THEN
             filtTest = 1
             FACTOR = factor * (T/(ABS(C) + 
     1                 filt_fac(1)*(1.0-ABS(C)))) ! "Wei" filter
          endif
          IF(FILTER(1) .eq. 1) THEN
             filtTest = 1
             FACTOR = factor * (T/(ABS(C) + 
     1                 filt_fac(1)))              ! Original filter
          ENDIF
          if(filtTest .eq. 0) then
             FACTOR = ABS(T*C)           ! default
          endif
          IF(CTFMODE.EQ.1) THEN             ! ADJUST PHASES AND AMPLITUDES
             IF(C.GT.0.0) FACTOR = -FACTOR
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
           ELSE IF(CTFMODE.EQ.2) THEN                     ! ADJUST PHASES ONLY
             IF(C.GT.0.0) THEN
               FFT(COUNT)   = -FFT(COUNT)
               FFT(COUNT+1) = -FFT(COUNT+1)
             ENDIF
           ELSE IF(CTFMODE.EQ.3) THEN                   ! ADJUST AMPLITUDES ONLY
             FFT(COUNT)   = FACTOR * FFT(COUNT)
             FFT(COUNT+1) = FACTOR * FFT(COUNT+1)
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
      END
