C  ****************************************************************************
C  *                                                                          *
C  *                      *****     IMGLIB.SUBS     *****                     *
C  *                                                                          *
C  *    THIS IS A LIBRARY OF SUBROUTINES WHICH ARE USED TO MANIPULATE "IMG"   *
C  *     ARRAY DATA.  THE OBJECT CODE FOR ALL THE ROUTINES ARE KEPT IN THE    *
C  *     LIBRARY DEXTRO3:TSBLIB.OLB                                           *
C  *                                                                          *
C  *    LIST OF SUBROUTINES:                                                  *
C  *                                                                          *
C  *      IMG_ADDIMG, IMG_ADDCONSTANT, IMG_AVERAGEFLT, IMG_BKGND,             *
C  *      IMG_CLEAR, IMG_COPY, IMG_DIF, IMG_DIVIDE, IMG_FFT,                  *
C  *      IMG_R4_FFT, IMG_FFT1, IMG_FFT2, IMG_FFT_FILL, IMG_R4_FFT_FILL,      *
C  *      IMG_FIND_PEAK, IMG_FIX, IMG_FLIP, IMG_GNOISE,                       *
C  *      IMG_I4_I2, IMG_LINGRAD, IMG_MAP, IMG_MASK,                          *
C  *      IMG_MEDIANFLT, IMG_MERGE, IMG_MULTIPLY, IMG_NOISE,                  *
C  *        IMG_POLAR, IMG_POW, IMG_ROT, IMG_SCALE, IMG_SHIFT,                  *
C  *        IMG_STATS, IMG_STATS_FLAG, IMG_STATS_NN, IMG_SYM, IMG_TRP,          *
C  *        IMG_TRUNCATE, IMG_TST_*, IMG_WEIGHTFLT, IMG_X_IMG,                  *
C  *        IMG_X_SUM,                                                          *
C  *                                                                          *
C  *    MAY 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  ****************************************************************************
C
C
C

C  **************************************************************
C  *                                                            *
C  *          *****     SUBROUTINE R4_IMG_ADDIMG     *****      *
C  *                                                            *
C  *    COMPUTE IMG1 + IMG2 AND STORE ANSWER IN IMG1.           *
C  *                                                            *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.    *
C  *    LAST MODIFIED  8-DEC-1989 BY T. BAKER                   *
C  **************************************************************
C
      SUBROUTINE R4_IMG_ADDIMG (IMG1, IMG2, NCOL, NROW)
C
      IMPLICIT NONE
C
      INTEGER*4 ARRAY_SIZE, NCOL, NROW, I
      real*4    IMG1(1), IMG2(1)
C -----------------------------------------------------------------------------
C      
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG1(I) = IMG1(I) + IMG2(I)         
      ENDDO
C
      RETURN
      END

C  *****************************************************************
C  *                                                               *
C  *         *****     SUBROUTINE IMGI4_ADDIMG     *****           *
C  *                                                               *
C  *    COMPUTE IMG1 + IMG2 AND STORE ANSWER IN IMG1.              *
C  *                                                               *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.       *
C  *    LAST MODIFIED  8-DEC-1989 BY T. BAKER                      *
C  *      Changed to I*4 format, 6-JUN-1994, D. Belnap, S. Walker    *
C  *****************************************************************
C
      SUBROUTINE IMGI4_ADDIMG (IMG1_I4, IMG2_I4, IMG2, NCOL, NROW)
C
      IMPLICIT NONE
C
      INTEGER*4 ARRAY_SIZE, NCOL, NROW, IMG1_I4(1)
      INTEGER*4 IMG2_I4(NCOL*NROW), I
      real*4    IMG2(1)
C -----------------------------------------------------------------------------
C      
      ARRAY_SIZE = NCOL*NROW
      DO I = 1,ARRAY_SIZE
        IMG2_I4(I) = IMG2(I)
      ENDDO
      DO I = 1,ARRAY_SIZE
        IMG1_I4(I) = IMG1_I4(I) + IMG2_I4(I)
      ENDDO
C
      RETURN
      END

C  **********************************************************************
C  *                                                                    *
C  *            *****     SUBROUTINE R4_IMG_ADDCONSTANT     *****       *
C  *                                                                    *
C  *    ADD A CONSTANT VALUE TO AN IMAGE.                               *
C  *                                                                    *
C  *    NOVEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER             *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).         *
C  **********************************************************************
C
      SUBROUTINE R4_IMG_ADDCONSTANT (IMG, NCOL, NROW, CONSTANT,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, ITMP, I, N1, NUMCHARI4
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, img(1)
      INTEGER*4 CONSTANT, ICOUNT
C -----------------------------------------------------------------------------
C      
      ARRAY_SIZE = NCOL*NROW
      ICOUNT     = 0
C
      DO I = 1,ARRAY_SIZE
        ITMP = IMG(I) + CONSTANT
        IF(IABS(ITMP).GT.I2_MAX) THEN
           ICOUNT  = ICOUNT + 1
           IMG(I) = ISIGN(1,ITMP)*I2_MAX
         ELSE
           IMG(I) = ITMP
        ENDIF
      ENDDO
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      IF(ICOUNT.NE.0) THEN
        PRINT *, '************************************'
        PRINT *, '* CAUTION: Overflow of INT*2 value *'
        PRINT *, '************************************'
        N1 = NUMCHARI4(ICOUNT)
        WRITE(*,'(A,I6,A)') ' There were ', ICOUNT, 
     1  ' over/under-flows'
      ENDIF
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE R4_IMG_AVERAGEFLT     *****           *
C  *                                                                          *
C  *    AVERAGING FILTER ROUTINE FOR IMAGE DATA.                              *
C  *                                                                          *
C  *    NCYCLE = NUMBER OF CYCLES OF FILTERING PERFORMED.                     *
C  *             (4 IS USUALLY ENOUGH)                                        *
C  *                                                                          *
C  *    ADAPTED FROM SUBROUTINE MEDAV WRITTEN BY W. GROCHULSKI.               *
C  *                                                                          *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                   *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C

      SUBROUTINE R4_IMG_AVERAGEFLT (IMG, NCOL, NROW, NCYCLE, IMG_MIN,
     $     IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)

      IMPLICIT NONE

      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, NCYCLE, I, J, K
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      INTEGER*4 C, D, E
      real*4, allocatable :: IMG2(:,:)
      INTEGER   ALLOC_STAT

      allocate(img2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_AVERAGEFLT'
      endif

      DO K = 1,NCYCLE
C  ****************************
C  *    AVERAGE ALONG ROWS    *
C  ****************************
        DO J = 1,NROW
          C            = IMG(1,J)
          D            = IMG(2,J)
          E            = IMG(NCOL,J)
          IMG2(1,J)    = (E+2.0*C+D)/4.0
          IMG2(NCOL,J) = (C+2.0*E+IMG(NCOL-1,J))/4.0
          DO I = 2,NCOL-1
            E         = IMG(I+1,J)
            IMG2(I,J) = (C+2.0*D+E)/4.0
            C         = D
            D         = E
          ENDDO
        ENDDO
C  ***********************************
C  *    NOW AVERAGE ALONG COLUMNS    *
C  ***********************************
        DO I = 1,NCOL
          C           = IMG2(I,1)
          D           = IMG2(I,2)
          E           = IMG2(I,NROW)
          IMG(I,1)    = (E+2.0*C+D)/4.0
          IMG(I,NROW) = (C+2.0*E+IMG2(I,NROW-1))/4.0
          DO J = 2,NROW-1
            E        = IMG2(I,J+1)
            IMG(I,J) = (C+2.0*D+E)/4.0
            C        = D
            D        = E
          ENDDO
        ENDDO
      ENDDO

      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
      deallocate(img2)

      RETURN
      END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE R4_IMG_BKGND     *****            *
C  *                                                                     *
C  *    COMPUTE THE BACKGROUND FOR "IMAGE" BY AVERAGING THE PERIMETER    *
C  *     INTENSITY VALUES.                                               *
C  *                                                                     *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER               *
C  *    LAST MODIFIED 10-DEC-1989  BY T. BAKER                           *
C  ***********************************************************************
C
      SUBROUTINE R4_IMG_BKGND (IMG, NCOL, NROW, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    BACK
      INTEGER*4 NCOL, NROW,  I
      real*4    IMG_BKG,  IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      BACK = 0.0
      DO I = 1,NCOL
        BACK = BACK + REAL(IMG(I,1))                  ! BOTTOM ROW
        BACK = BACK + REAL(IMG(I,NROW))            ! TOP ROW
      ENDDO
C
      DO I = 2,(NROW-1)
        BACK = BACK + REAL(IMG(1,I))                  ! LEFT COLUMN
        BACK = BACK + REAL(IMG(NCOL,I))            ! RIGHT COLUMN
      ENDDO
C
      BACK    = BACK/(2*NCOL+2*NROW-4)
      IMG_BKG = BACK
      WRITE(*,1000) IMG_BKG
      RETURN
C
 1000 FORMAT('+Image background value =',f12.5,'            ')
      END

C  **************************************************************
C  *                                                            *
C  *           *****     SUBROUTINE R4_IMG_CLEAR     *****         *
C  *                                                            *
C  *    ZEROES THE "IMAGE" ARRAY                                *
C  *                                                            *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER      *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM). *
C  **************************************************************
C
      SUBROUTINE R4_IMG_CLEAR (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, 
     1   IMG_BKG, IMG_AVG, IMG_STD)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, I,  ARRAY_SIZE
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(1)
      INTEGER*4 ZERO
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      ZERO       = 0
C
      DO I = 1,ARRAY_SIZE
        IMG(I) = ZERO
      ENDDO
C  ***************************************
C  *    REINITIALIZE IMG_* PARAMETERS    *
C  ***************************************
      IMG_MIN = 0.0
      IMG_MAX = 0.0
      IMG_BKG = 0.0
      IMG_AVG = 0.0
      IMG_STD = 0.0
C
      RETURN
      END

C  ************************************************************
C  *                                                          *
C  *          *****     SUBROUTINE R4_IMG_COPY     *****      *
C  *                                                          *
C  *    COPY PART OF ONE IMG ARRAY INTO ANOTHER.              *
C  *                                                          *
C  *    FEBRUARY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.  *
C  *    LAST MODIFIED 14-FEB-1992 BY T. BAKER.                *
C  ************************************************************
C
      SUBROUTINE R4_IMG_COPY (IMG1, NCOL1, NROW1, IX1, IX2, IY1, IY2,
     1 IMG2, NCOL2, NROW2, IX3, IY3)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL1, NROW1, NCOL2, NROW2, I, J
      INTEGER*4 IX, IX1, IX2, IX3, IY, IY1, IY2, IY3
      INTEGER*4 X1, X2, X3, X4, Y1, Y2, Y3, Y4
      real*4    IMG1(NCOL1,NROW1), IMG2(NCOL2,NROW2)
C  ----------------------------------------------------------------------------
C
C  ***************
C  *    TRAPS    *
C  ***************
      X1 = IX1
      X2 = IX2
      X3 = IX3
      X4 = IX3 + (IX2 - IX1 + 1)
      Y1 = IY1
      Y2 = IY2
      Y3 = IY3
      Y4 = IY3 + (IY2 - IY1 + 1)
      IF(IX1.LE.0)     X1 = 1
      IF(IX2.GT.NCOL1) X2 = NCOL1
      IF(IY1.LE.0)     Y1 = 1
      IF(IY2.GT.NROW1) Y2 = NROW1
      DO J = Y1,Y2
        IY = Y3 + (J - Y1)
        DO I = X1,X2
          IX = X3 + (I - X1)
          IMG2(IX,IY) = IMG1(I,J)
        ENDDO
      ENDDO
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *             *****  SUBROUTINE R4_IMG_DIF  *****              *
C  *                                                           *
C  *    COMPUTE IMG1 - IMG2 AND STORE ANSWER IN IMG1           *
C  *                                                           *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER         *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).*
C  *************************************************************
C
      SUBROUTINE R4_IMG_DIF (IMG1, IMG2, NCOL, NROW, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG1(1), IMG2(1)
C -----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG1(I) = IMG1(I) - IMG2(I)
      ENDDO
      CALL R4_IMG_STATS (IMG1, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *             *****  SUBROUTINE R4_IMG_DIVIDE  *****           *
C  *                                                           *
C  *    DIVIDE ENTIRE IMAGE ARRAY BY "DIV".                    *
C  *                                                           *
C  *    DECEMBER 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER     *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).*
C  *************************************************************
C
      SUBROUTINE R4_IMG_DIVIDE (IMG, NCOL, NROW, DIV, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, DIV
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG,  IMG(1)
C -----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG(I) = NINT(REAL(IMG(I))/DIV)
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *              *****     SUBROUTINE R4_IMG_FFT     *****              *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY AND COMPUTE FFT.    *
C  *                                                                  *
C  *    JUNE 1991  PURDUE UNIVERSITY VAX 8550, T. BAKER.              *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).             *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE R4_IMG_FFT (IMG, NCOL, NROW, FFT, IDIM1, IDIM2,
     1 IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2,  EM_WRT
      real*4    IMG_BKG, IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      CALL R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, IMG_BKG, 
     1 FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
      CALL FFT_2D (FFT, IDIM1, IDIM2)
C
      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE IMG_R4_FFT     *****                *
C  *                                                                         *
C  *    LOAD NCOL X NROW REAL*4 "IMAGE" INTO "FFT" ARRAY AND COMPUTE FFT.    *
C  *                                                                         *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
C  *    MODIFIED 31-AUG-1999 BY TSB/RWA. (Patterned after IMG_FFT).          *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).              *
C  ***************************************************************************
C
      SUBROUTINE IMG_R4_FFT (IMG_R4, NCOL, NROW, FFT, IDIM1, IDIM2,
     1 IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2,  EM_WRT
      real*4    IMG_BKG
      REAL*4    FFT(2*IDIM1,IDIM2), FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      REAL*4    IMG_R4(NCOL,NROW)
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      CALL IMG_R4_FFT_FILL (IMG_R4, NCOL, NROW, FFT, IDIM1, IDIM2, 
     1 IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
      CALL FFT_2D (FFT, IDIM1, IDIM2)
C
      RETURN
      END

C  *****************************************************************
C  *                                                               *
C  *             *****     SUBROUTINE R4_IMG_FFT1     *****           *
C  *                                                               *
C  *    FIRST DIMENSION OF FFT (UP TO 1024 COLUMNS)                *
C  *                                                               *
C  *    THE UNIQUE PART OF THE FIRST DIMENSION OF THE FFT IS       *
C  *     STORED IN CORE                                            *
C  *****************************************************************
C
      SUBROUTINE R4_IMG_FFT1 (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, 
     $     COLUMN, FFT_FZERO, IMG_BKG)
C     
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2
      real*4    IMG_BKG,  IMG(NCOL,NROW)
      INTEGER*4 J, J1, J2, K, L
      REAL*4    COLUMN(IDIM2*2), FFT(2*IDIM1,((IDIM2/2)+1)), WORK
      REAL*4    FFT_FZERO
C  -----------------------------------------------------------------------------
C
      FFT_FZERO = 0.0
C
      DO J = 1,IDIM1
        IF(J.LE.NCOL) THEN
          DO K = 1,IDIM2*2                           ! ZERO "COLUMN" ARRAY
            COLUMN(K) = 0.0
          ENDDO
          DO K = 1,NROW            ! COPY COLUMN OF IMG INTO "COLUMN" ARRAY
            L          = 2*K-1
            COLUMN(L)  = IMG(J,K) - IMG_BKG
            FFT_FZERO  = FFT_FZERO + COLUMN(L)
          ENDDO
          CALL FOURT (COLUMN, IDIM2, 1, 1, 0, WORK)     ! TRANSFORM THE COLUMN
          J1 = J*2-1                ! STORE UNIQUE PART OF THE TRANSFORM IN "FFT"
          J2 = J1+1
          DO K = 1,(IDIM2/2)+1
            FFT(J1,K) = COLUMN(2*K-1)
            FFT(J2,K) = COLUMN(2*K)
          ENDDO
        ENDIF
      ENDDO
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                 *****     SUBROUTINE R4_IMG_FFT2     *****                *
C  *                                                                        *
C  *    THIS COMPUTES THE SECOND DIMENSION OF THE FFT ((IDIM2/2)+1 ROWS)    *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).              *
C  **************************************************************************
C
      SUBROUTINE R4_IMG_FFT2 (NCOL, NROW, IDIM1, IDIM2, ROW, FFT, FSQ,
     1 FFT_FILENAME, FFT_TITLE, FFT_ORIGX, FFT_ORIGY, FSQ_AVG, 
     2 FSQ_STD, FSQ_MAX, FFT_FZERO, FILE_ID, IMG_BKG)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      COMMON    NREC2, LUNIN, LUNOUT
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, LUNIN, LUNOUT, NREC2, LRECL
      INTEGER*4 FILE_ID, FFT_TITLE(18)
      real*4     IMG_BKG
      INTEGER*4 I, J, K, L, NBKG
      REAL*4    FFT(2*IDIM1,((IDIM2/2)+1)), ROW(IDIM1*2), SUM, SUMSQ
      REAL*4    FFT_ORIGX, FFT_ORIGY, FSQ_AVG, FSQ_STD, FSQ_MAX
      REAL*4    FSQ, DIV, VAR, WORK, FFT_FZERO
      CHARACTER FFT_FILENAME*64
C  -----------------------------------------------------------------------------
C
      SUM     =    0.0
      SUMSQ   =    0.0
      FSQ_MAX = R4_MIN
      NREC2   =      4
C
      LRECL = 2*IDIM1
      OPEN (UNIT=LUNOUT, FILE=FFT_FILENAME, STATUS='NEW',
     1     ACCESS='DIRECT', RECL=LRECL) !RSS ASSOCIATEVARIABLE=NREC2)
      FILE_ID = 1
C
      DO I = 1,(IDIM2/2)+1                  ! CALCULATE TRANSFORM ROW BY ROW
        DO L = 1,2*IDIM1
          ROW(L) = FFT(L,I)                  ! LOAD A ROW FROM "FFT" INTO "ROW"
        ENDDO
        CALL FOURT (ROW, IDIM1, 1, 1, 1, WORK)           ! TRANSFORM THE ROW
        DO L = 1,IDIM1*2,2                 ! CALCULATE FSQ_MAX,FSQ_AVG,FSQ_STD
          FSQ     = ROW(L)**2 + ROW(L+1)**2
          FSQ_MAX = AMAX1(FSQ_MAX,FSQ)
          SUM     = SUM+FSQ
          SUMSQ   = SUMSQ+FSQ*FSQ
        ENDDO
        WRITE(LUNOUT,REC=NREC2) ROW
      ENDDO
C  ***************************************************************************
C  *    STORE IMPORTANT TRANSFORM PARAMETERS IN FIRST RECORD OF TRANSFORM    *
C  *    FOR USE BY OTHERS PROGRAMS                                           *
C  ***************************************************************************
      FFT_ORIGX = (NCOL-1)/2.0
      FFT_ORIGY = (NROW-1)/2.0
      DIV       = REAL(IDIM1)*REAL((IDIM2/2)+1)
      FSQ_AVG   = SUM/DIV
      VAR       = (SUMSQ-(SUM**2.0)/DIV)/(DIV-1.0)
        IF(VAR.GE.0.0) FSQ_STD = SQRT(VAR)
      NBKG = IMG_BKG
      WRITE(LUNOUT,REC=1) FILE_ID
      WRITE(LUNOUT,REC=2) FFT_TITLE
      WRITE(LUNOUT,REC=3) FFT_FZERO, FFT_ORIGX, FFT_ORIGY, 1, IDIM1, 
     1               IDIM2, FSQ_AVG, FSQ_STD, FSQ_MAX, NBKG
C
      PRINT *, '*************************'
      PRINT *, '* Wake up, it''s done... *'
      PRINT *, '*************************'
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE R4_IMG_FFT_FILL     *****           *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *                                                                  *
C  *    BEWARE: THE DATA ***MUST*** BE PROPERLY FLOATED BEFORE THE    *
C  *            DATA ARE FOURIER TRANSFORMED.                         *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).             *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).       *
C  ********************************************************************
C
      SUBROUTINE R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2,
     1 IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J,  EM_WRT
      real*4    IMG_BKG
      real*4    IMG(NCOL,NROW)
      REAL*4    FFT(2*IDIM1,IDIM2), TMP, FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      IF(FFT_ORIGX.EQ.0.0 .AND. FFT_ORIGY.EQ.0.0) THEN
        FFT_ORIGX = (NCOL-1)/2.0
        FFT_ORIGY = (NROW-1)/2.0
      ENDIF
      FFT_FZERO = 0.0
      IF(EM_WRT.EQ.1) THEN
        PRINT *, '*****************************'
        PRINT *, '* Load IMAGE into FFT array *'
        PRINT *, '*****************************'
      ENDIF
C
      FFT_FULL = .TRUE.
      CALL FFT_CLEAR (FFT, IDIM1, IDIM2, FFT_FULL)        ! ZERO THE FFT ARRAY
C
      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = IMG(I,J) - IMG_BKG         ! FLOATING OCCURS HERE
          FFT((2*I)-1,J) = TMP
          FFT_FZERO      = FFT_FZERO + TMP
        ENDDO
      ENDDO
C
      IF(EM_WRT.EQ.1) WRITE(*,1000) FFT_FZERO
 1000      FORMAT(' F-ZERO OF FLOATED IMAGE =',E12.3,'            ')
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE IMG_R4_FFT_FILL     *****             *
C  *                                                                          *
C  *    LOAD NCOL X NROW REAL*4 "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT       *
C  *                                                                          *
C  *    BEWARE: THE DATA ***MUST*** BE PROPERLY FLOATED BEFORE THE            *
C  *            DATA ARE FOURIER TRANSFORMED.                                 *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 31-AUG-1999 BY TSB/RWA. (Patterned after IMG_FFT_FILL).      *
C  *    LAST MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).               *
C  ****************************************************************************
C
      SUBROUTINE IMG_R4_FFT_FILL (IMG_R4, NCOL, NROW, FFT, IDIM1, 
     1 IDIM2, IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, 
     2 EM_WRT)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J,  EM_WRT
      real*4    IMG_BKG
      REAL*4    FFT(2*IDIM1,IDIM2), TMP, FFT_ORIGX, FFT_ORIGY, FFT_FZERO
      REAL*4    IMG_R4(NCOL,NROW)
      LOGICAL   FFT_FULL
C  -----------------------------------------------------------------------------
C
      IF(FFT_ORIGX.EQ.0.0 .AND. FFT_ORIGY.EQ.0.0) THEN
        FFT_ORIGX = (NCOL-1)/2.0
        FFT_ORIGY = (NROW-1)/2.0
      ENDIF
      FFT_FZERO = 0.0
      IF(EM_WRT.EQ.1) THEN
        PRINT *, '*****************************'
        PRINT *, '* Load IMAGE into FFT array *'
        PRINT *, '*****************************'
      ENDIF
C
      FFT_FULL = .TRUE.
      CALL FFT_CLEAR (FFT, IDIM1, IDIM2, FFT_FULL)        ! ZERO THE FFT ARRAY
C
      DO J = 1,NROW
        DO I = 1,NCOL
          TMP            = IMG_R4(I,J) - IMG_BKG       ! FLOATING OCCURS HERE
          FFT((2*I)-1,J) = TMP
          FFT_FZERO      = FFT_FZERO + TMP
        ENDDO
      ENDDO
C
      IF(EM_WRT.EQ.1) WRITE(*,1000) FFT_FZERO
 1000      FORMAT(' F-ZERO OF FLOATED IMAGE =',E12.3,'            ')
C
      RETURN
      END

C  **************************************************************
C  *                                                *
C  *            ***** SUBROUTINE R4_IMG_FIND_PEAK *****            *
C  *                                                *
C  *            Finds a maximum in a REAL*4 array, and returns  *
C  *      its coordinates.                              *
C  *                                                *
C  *      CREATED: June 6, 1991  by nrd@bragg.bio.purdue.edu      * 
C  **************************************************************
C
      SUBROUTINE R4_IMG_FIND_PEAK (R4_IMG, NCOL, NROW, X, Y, MAX)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW, X, Y, I, J
      REAL*4        R4_IMG(NCOL, NROW), MAX
C  --------------------------------------------------------------------
      MAX = 0.0
      DO J=1,NROW
        DO I=1,NCOL
          IF (R4_IMG(I,J).GT.MAX) THEN
            X=I
            Y=J
            MAX = R4_IMG(I,J)
          ENDIF
        ENDDO
      ENDDO 
      WRITE (6,'(A)') ' ************************'
      WRITE (6,'(A,F5.3,A)') ' *  CORRELATION : ', MAX, ' *'
      WRITE (6,'(A)') ' ************************'
C
      RETURN
      END

C  *****************************************************************
C  *                                                               *
C  *            *****     SUBROUTINE R4_IMG_FIX      *****            *
C  *                                                               *
C  *    THIS ROUTINE HAS FIVE OPTIONS:                             *
C  *                                                               *
C  *    (A) REPLACE VALUES INSIDE OR OUTSIDE CIRCULAR BOUNDARY     *
C  *        WITH USER-DEFINED VALUE (IDENS)                        *
C  *                                                               *
C  *    (B) REPLACE VALUES OUTSIDE A "WINDOW" (LOI, HII) WITH      *
C  *        IDENS (DEFAULT = IMG_AVG)                              *
C  *                                                               *
C  *    (C) DECREASE THE NUMBER OF GREY LEVELS                     *
C  *                                                               *
C  *    (D) ADD A CONSTANT TO THE IMAGE                            *
C  *                                                               *
C  *    (F) SUBTRACT A LINEAR GRADIENT FROM THE IMAGE              *
C  *                                                               *
C  *    (G) APPLY MEDIAN FILTERING TO THE IMAGE.                   *
C  *                                                               *
C  *    (H) APPLY SMOOTHING FILTER TO IMAGE.                       *
C  *                                                               *
C  *    MARCH 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER.        *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).    *
C  *****************************************************************
C
      SUBROUTINE R4_IMG_FIX (IMG, NCOL, NROW, IERR, IKEY, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    R, RSQ, XCEN, YCEN, XSQ, YSQ, TEST, STEP_SIZE
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, IRTMP, N1, N2, NUMCHARI2, NUMCHARI4
      INTEGER*4 FILE_ID
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      real*4    INT1_OLD, INT2_OLD, INT1_NEW, INT2_NEW
      integer*4 NCYCLE, IERR
      INTEGER*4 I, J, K, IR, LOI, HII, NLEVELS, LEVEL, LUTLOI, LUTHII
      INTEGER*4 IX, IY, ISIDE, IDENS
      CHARACTER*1 IKEY
C  ----------------------------------------------------------------------------
C
      LUTLOI =   0
      LUTHII = 255
      IF(IKEY.EQ.'A') THEN
         IX = (NCOL+1)/2
         IY = (NROW+1)/2
         IF(IX.LT.IY) THEN
            IR = IX - 1
          ELSE
            IR = IY - 1      
         ENDIF
         WRITE (6,'(A,I3,A,I3,A,I3,A)') 'CENTER AND RADIUS <CR>=',
     1        IX, ',' , IY, ',', IR, ': '
         READ (*,*) IX, IY, IRTMP
         IF(IX.EQ.0 .AND. IY.EQ.0) THEN
            IX = (NCOL+1)/2
            IY = (NROW+1)/2
         ENDIF
         IF(IRTMP.LE.0) THEN
            R = IR
          ELSE
            R = IRTMP
         ENDIF
  100         WRITE(*,'(2A)') 'REPLACE VALUES OUTSIDE<CR> OR INSIDE(1)',
     1   ' BOUNDARY?: '
         READ (*,*,ERR=100) ISIDE
         IF (ISIDE.EQ.1) THEN
            ISIDE = -1                  ! INSIDE BOUNDARY
          ELSE
            ISIDE = 0                        ! OUTSIDE BOUNDARY
         ENDIF
         WRITE(*,'(/,A,f12.5,/)') ' Note that IMG_BKG = ', IMG_BKG
 200     WRITE(*,'(A)') 'ENTER IDENS (I FORMAT): '
         READ (*,*,ERR=200) IDENS
C
         RSQ  = R * R
         XCEN = IX
         YCEN = IY
         DO J = 1,NROW
           YSQ = (REAL(J)-YCEN)**2
           DO I = 1,NCOL
             XSQ = (REAL(I)-XCEN)**2
             TEST = XSQ + YSQ
             IF(TEST.GT.RSQ .AND. ISIDE.GE.0) IMG(I,J) = IDENS
             IF(TEST.LE.RSQ .AND. ISIDE.LT.0) IMG(I,J) = IDENS
           ENDDO
         ENDDO
C
      ELSE IF (IKEY.EQ.'B') THEN
 300     WRITE (6,'(A)') 'RANGE TO PRESERVE (LOI,HII): '
         READ (*,*,ERR=300) LOI, HII
 400     WRITE (6,'(A,I3,A)') 'IDENS (<CR> = ', NINT(IMG_AVG), '): '
         READ (*,*,ERR=400) IDENS
         IF(IDENS.EQ.0) IDENS = NINT(IMG_AVG)
            DO J = 1,NROW
           DO I = 1,NCOL
             IF(IMG(I,J).LE.LOI .OR. IMG(I,J).GE.HII) THEN
               IMG(I,J) = IDENS
             ENDIF
           ENDDO
         ENDDO
C
       ELSE IF (IKEY.EQ.'C') THEN
 500      WRITE (6,'(A)') 'NUMBER OF GREY-LEVELS (2-256): '
         READ (*,*,ERR=500) NLEVELS
         IF(NLEVELS.EQ.0.OR.NLEVELS.EQ.256) RETURN
         IF(NLEVELS.LE.1.OR.NLEVELS.GT.256) GO TO 500
         STEP_SIZE = REAL(LUTHII-LUTLOI)/REAL(NLEVELS-1)
         DO J= 1,NROW
           DO I = 1,NCOL
             LEVEL    = NINT(REAL(IMG(I,J))/STEP_SIZE)
             IMG(I,J) = INT(LEVEL*STEP_SIZE) + LUTLOI
           ENDDO
         ENDDO
       ELSE IF (IKEY.EQ.'D') THEN
  600         WRITE(*,'(A)') 'VALUE TO ADD TO ALL IMAGE PIXELS: '
         READ (*,*,ERR=600) K
         CALL R4_IMG_ADDCONSTANT (IMG, NCOL, NROW, K, IMG_MIN,
     1      IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
  610         INT1_NEW = IMG_MIN
         INT2_NEW = IMG_MAX
         N1       = NUMCHARI2(LUTLOI)
         N2       = NUMCHARI2(LUTHII)
         WRITE(*,'(A)') 'RESCALE THE IMAGE? NO<CR> YES(1): '
         READ (*,*) I
 619     IF(I.EQ.1) THEN
 620        WRITE(*,'(2A,I7,A,I7,2A)') ' ENTER NEW MIN, MAX',
     1           ' LIMITS (<CR>=', LUTLOI, ',', LUTHII, '): '
           READ (*,*,ERR=620) INT1_NEW, INT2_NEW
           IF(INT1_NEW.EQ.0 .AND. INT2_NEW.EQ.0) THEN
             INT1_NEW = LUTLOI
             INT2_NEW = LUTHII
           ENDIF
           INT1_OLD = IMG_MIN
           INT2_OLD = IMG_MAX
           CALL R4_IMG_SCALE (IMG, NCOL, NROW, INT1_OLD, INT2_OLD,
     1     INT1_NEW, INT2_NEW, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD,
     2        IMG_BKG)
         ENDIF
C
         IF(INT1_NEW.LT.LUTLOI .OR. INT2_NEW.GT.LUTHII) THEN
 630        WRITE(*,'(2A)') 'ARE YOU SURE YOU DO NOT WANT TO RESCALE?',
     1     ' YES<CR> NO(1): '
           READ (*,*,ERR=630) I
           IF(I.EQ.1) GO TO 619
         ENDIF
      ELSE IF (IKEY.EQ.'F') THEN
 700     WRITE(*,'(2A)') 'COMPUTE GRADIENT OVER ENTIRE IMAGE<CR> OR',
     1        ' ONLY WITHIN CIRCULAR BOUNDARY(1): '
         READ (*,*,ERR=700) I
         IF(I.NE.1) THEN
            CALL R4_IMG_LINGRAD (IMG, NCOL, NROW, 0, FILE_ID, IMG_MIN,
     1         IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
          ELSE
            IRTMP = MIN(NCOL,NROW)/2
            N1    = NUMCHARI4(IRTMP)
 750        WRITE(*,'(A,I7,A)') 'ENTER RADIUS (<CR>=', IRTMP, '): '
            READ (*,*,ERR=750) I
            IF(I.GT.IRTMP) THEN
              PRINT *, '********************************'
              PRINT *, '* Radius exceeds allowed limit *'
              PRINT *, '********************************'
              GO TO 750
            ENDIF
            IF(I.GT.0) IRTMP = I
            CALL R4_IMG_LINGRAD (IMG, NCOL, NROW, IRTMP, FILE_ID,
     1           IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
         ENDIF
       ELSE IF (IKEY.EQ.'G') THEN
  800         WRITE(*,'(A)') 'NUMBER OF CYCLES OF MEDIAN FILTERING: '
         READ (*,*,ERR=800) NCYCLE
         CALL R4_IMG_MEDIANFLT (IMG, NCOL, NROW, NCYCLE, IMG_MIN,
     1      IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
       ELSE IF (IKEY.EQ.'H') THEN
  900         WRITE(*,'(A)') 'NUMBER OF CYCLES FOR SMOOTHING: '
         READ (*,*,ERR=900) NCYCLE
         CALL R4_IMG_AVERAGEFLT (IMG, NCOL, NROW, NCYCLE, IMG_MIN,
     1      IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
       ELSE
         IERR = 1
      ENDIF
C
      RETURN
      END

C  ****************************************************************
C  *                                                              *
C  *            *****     SUBROUTINE R4_IMG_FLIP     *****           *
C  *                                                              *
C  *    INVERT "IMAGE" ABOUT X AND/OR Y                           *
C  *                                                              *
C  *    NOVEMBER 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER     *
C  *    AUGUST 1987  PURDUE UNIVERSITY 8550, T. BAKER             *
C  *    LAST MODIFIED 10-DEC-1989  BY T. BAKER                    *
C  ****************************************************************
C
      SUBROUTINE R4_IMG_FLIP (IMG1, IMG2, NCOL, NROW, XFLIP, YFLIP)
C
      IMPLICIT NONE
C
      INTEGER*4 NCOL, NROW
      real*4    IMG1(NCOL,NROW), IMG2(NCOL,NROW)
      INTEGER*4 I, II, ISTEP, ISTART, J, JJ, JSTEP, JSTART, ITMP
      LOGICAL   XFLIP, YFLIP
C  ----------------------------------------------------------------------------
C
      IF(XFLIP) THEN
         ISTEP  = -1                  ! FLIP IMAGE IN X
         ISTART = NCOL+1            ! [X = -X]
       ELSE
         ISTEP  = 1                  ! NO FLIP IN X
         ISTART = 0                  ! [X = X]
      ENDIF
C
      IF(YFLIP) THEN
         JSTEP  = -1                  ! FLIP IMAGE IN Y
         JSTART = NROW+1            ! [Y = -Y]
       ELSE
         JSTEP  = 1                  ! NO FLIP IN Y
         JSTART = 0                  ! [Y = Y]
      ENDIF
C
      JJ = JSTART
      DO J = 1,NROW
        JJ = JJ+JSTEP
        II = ISTART
        DO I = 1,NCOL
          II        = II + ISTEP
          IMG2(I,J) = IMG1(II,JJ)
        ENDDO
      ENDDO
C
      DO J = 1,NROW
        DO I = 1,NCOL
          ITMP      = IMG1(I,J)
          IMG1(I,J) = IMG2(I,J)
          IMG2(I,J) = ITMP
        ENDDO
      ENDDO
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *         *****     SUBROUTINE R4_IMG_GNOISE     *****         *
C  *                                                           *
C  *    ADD RANDOM GAUSSIAN NOISE TO "IMAGE"                   *
C  *                                                           *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER      *
C  *    LAST MODIFIED 11-AUG-1999  BY TSB (Fix ISEED to be     *
C  *     generated only on 1st call to this troutine.          *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).*
C  *************************************************************
C
      SUBROUTINE R4_IMG_GNOISE (IMG, NCOL, NROW, STD, GAUSS, ICOUNT,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
Ca      INTEGER*4 ISEED, LIM, N, NUMCHARI4, ICOUNT, GAUSS(ICOUNT)
      INTEGER*4 ISEED, LIM, N, NUMCHARI4, ICOUNT, SEED_COUNT
      REAL*4    GAUSS(ICOUNT), IMG_AVG, IMG_STD, STD, R4ICOUNT, RAN
      INTEGER*4 NCOL, NROW, IARRAY, ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(1)
      INTEGER*4 J
      DATA      SEED_COUNT/0/
      SAVE      ISEED, SEED_COUNT
C  ----------------------------------------------------------------------------
C
      SEED_COUNT = SEED_COUNT + 1
      IF(SEED_COUNT.EQ.1) CALL SEED (ISEED)
      ARRAY_SIZE = NCOL*NROW
      LIM        = NINT(STD)
      N          = NUMCHARI4(LIM)
      WRITE(*,'(A,I5)') ' Add Gaussian noise with STD of +/-', LIM
C
      R4ICOUNT = REAL(ICOUNT)
C
      DO I = 1,ARRAY_SIZE
        IARRAY = RAN(ISEED)*R4ICOUNT + 1
Ca        IMG(I) = IMG(I) + GAUSS(IARRAY)
        IMG(I) = IMG(I) + (GAUSS(IARRAY) * STD)
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  ****************************************************************
C  *                  SUBROUTINE IMG_I4_I2                    *
C  *                                                  *
C  *      Converts and I*2 image into I*4 format and vice versa.    *
C  *                                                  *
C  *      CONVERT_STRING = "I2I4" if converting from I*2 to I*4     *
C  *                     = "I4I2" if converting from I*4 to I*2     *
C  *                                                  *
C  *      Written 6 June 1994 by D. Belnap & S. Walker              *
C  ****************************************************************
C
      SUBROUTINE IMG_I4_I2(CONVERT_STRING, IMG_I2, IMG_I4, NCOL, NROW)
C
      IMPLICIT NONE
C
      INTEGER*4   NCOL, NROW, IMG_I4(NCOL,NROW), I, J, IMG_I4_MAX
      REAL*4      SCALE
      INTEGER*2   IMG_I2(NCOL,NROW), I4I2
      CHARACTER*4 CONVERT_STRING
C
      IF (CONVERT_STRING.EQ.'I2I4') THEN             !Convert from I*2 to I*4
         DO I = 1,NCOL
           DO J = 1,NROW
             IMG_I4(I,J) = IMG_I2(I,J)
           ENDDO
         ENDDO
       ELSE IF (CONVERT_STRING.EQ.'I4I2') THEN       !Convert from I*4 to I*2
         IMG_I4_MAX = 0
         DO I = 1,NCOL
           DO J = 1,NROW
             IMG_I4_MAX = MAX(IMG_I4_MAX,IMG_I4(I,J))
           ENDDO
         ENDDO
         IF (IMG_I4_MAX.GT.32767) THEN
           SCALE = 32767.0/IMG_I4_MAX
           DO I = 1,NCOL
             DO J = 1,NROW
             IMG_I4(I,J) = INT(SCALE * IMG_I4(I,J))
             ENDDO
           ENDDO
         ENDIF
         DO I = 1,NCOL
           DO J = 1,NROW
             IMG_I2(I,J) = I4I2(IMG_I4(I,J))
           ENDDO
         ENDDO
      ENDIF
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE R4_IMG_LINGRAD     *****               *
C  *                                                                          *
C  *      REMOVE LINEAR GRADIENT FROM IMG(NCOL,NROW) IMAGE ARRAY.               *
C  *                                                                          *
C  *    THIS SUBROUTINE WAS ORIGINALLY PART OF MDPP AND WAS WRITTEN (I        *
C  *     PRESUME) BY ROSS SMITH.  IT WAS REWRITTEN AROUND 1980 AT NIH FOR     *
C  *     PDP 11S, AND SUBSEQUENTLY ADAPTED TO THE VAX (WITH VERY LITTLE       *
C  *     ADDITIONAL MODIFICATION). -BENES TRUS                                *
C  *                                                                          *
C  *      NEW ADDITION: POINTS OUTSIDE OF A CIRCLE IN THE IMAGE MAY BE          *
C  *                  EXCLUDED.                                      *
C  *                                                                          *
C  *      IRAD    = RADIUS OF DATA TO INCLUDE IN GRADIENT CALCULATION.             *
C  *            = 0 IMPLIES CALCULATING A LINEAR GRADIENT OVER THE WHOLE      *
C  *                IMAGE. USES BIMG_CURRENT (CURRENT IMAGE IN PACK FILE      *
C  *                BEING WORKED ON).                                         *
C  *                                                                          *
C  *    16-JANUARY 1990  NIH, BENES TRUS.                                     *
C  *      MODIFIED: 20th June, 1990  BY NRD@BRAGG.BIO.PURDUE.EDU                *
C  *    LAST MODIFIED 16-DEC-1997  BY T. BAKER (RID OF EM.CMM).               *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_LINGRAD (IMG, NCOL, NROW, IRAD, FILE_ID,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
CCC
CCC  NEED TO PASS IN BIMG_BKGS, BIMG_CURRENT???
CCC
C
      IMPLICIT NONE 
C
      LOGICAL*1 FN(30), IAA(1024)
      INTEGER*4 IRAD, NCOL, NROW, I, J, IC, JC, BIMG_NMAX
      PARAMETER (BIMG_NMAX=10000)
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG
      real*4    IMG(NCOL, NROW), NEW_BKG
      INTEGER*4 FNCNT,TITLEN,TITLE(60)
      INTEGER*4 NFIT, NNFIT, ISOL, IDSOL, FILE_ID
      INTEGER*4 INUM, KWA(20), BIMG_CURRENT, BIMG_BKGS(BIMG_NMAX)
      REAL*4    ARRAY(1024), DIST, XRAD, TMP, STD, AVG, STD2
      REAL*4        VAR, SUMSQ, TOTPTS, BACK, YSQ, RSQ1, RSQ2
      REAL*4    IMG_AVG, IMG_STD
      DOUBLE PRECISION    Q(20,20),G(20),DET,T(20,21),XPIX
      COMMON /FITTER/Q,G
      COMMON /FITOUT/T
      COMMON /PARAM/NFIT,NNFIT
C ----------------------------------------------------------------------
C
      IF (IRAD.GT.0 .AND. FILE_ID.EQ.-10) THEN
        IMG_BKG = BIMG_BKGS(BIMG_CURRENT)
      ENDIF
      RSQ1 = (REAL(IRAD)-1.0)**2
      RSQ2 = (REAL(IRAD))**2
      INUM=0
      NFIT=3
      NNFIT=(NFIT+1)
c
      DO I=1,NFIT
        G(I)=0.D0
        DO J=1,NFIT
          Q(I,J)=0.D0
        ENDDO
      ENDDO
      JC=NROW/2+1
      IC=NCOL/2+1
      DO J=1,NROW
        YSQ = (REAL(J-JC))**2
        DO I=1,NCOL
         XPIX=REAL(IMG(I, J))
         IF(IRAD.EQ.0) THEN
            GOTO 27
          ELSE
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST.GT.RSQ2) GOTO 7
         ENDIF
27        CONTINUE
        CALL TRUS_PLYFIT (I, J, XPIX)
        INUM=INUM+1
7       ENDDO
      ENDDO
      DO I=1,NFIT
        T(I,NNFIT)=G(I)
        DO J=1,NFIT
          T(I,J)=Q(I,J)
        ENDDO
      ENDDO
C
      CALL TRUS_MATDBL (ISOL, IDSOL, NFIT, -NNFIT, T, 20, KWA, DET)
C  ****************************************************************************
C  *    USE PLYOUT SUBROUTINE TO MODIFY ALL PIXELS OF IMG ARRAY (INSIDE RSQ2).*
C  *                                                                          *
C  *    IRAD<>0 : REMOVE GRADIENT FROM DATA INSIDE RSQ2, CALCULATE NEW_BKG.   *
C  ****************************************************************************
      BACK   = 0.0
      TOTPTS = 0.0
      SUMSQ  = 0.0
      IF (IRAD.NE.0) THEN
        DO J=1,NROW
          YSQ = (REAL(J-JC))**2
          DO I=1,NCOL
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST.GT.RSQ2) GOTO 40
            XPIX = REAL(IMG(I,J))
            CALL TRUS_PLYOUT (I, J, XPIX)
            IMG(I,J) = NINT(XPIX)            ! IMG NOW FLOATED AROUND 0     
            IF (DIST.GE.RSQ1) THEN
            TMP    = IMG(I,J)            ! NEW_BKG WILL BE NEAR 0
              BACK   = BACK+TMP
              TOTPTS = TOTPTS+1.0
              SUMSQ  = SUMSQ+TMP*TMP
            ENDIF      
40          ENDDO
        ENDDO
C
        STD     = 0.0
        AVG     = BACK/TOTPTS
        VAR     = (SUMSQ/TOTPTS) - (AVG*AVG)
        IF(VAR.GE.0.0) STD = SQRT(VAR)
C  *****************************************************************
C  *    THROW OUT DATA POINTS OUTSIDE OF 2*STD FROM CALCULATION    *
C  *****************************************************************
         BACK   = 0.0
         TOTPTS = 0.0
         SUMSQ  = 0.0
        IF (STD.GT.0.0) THEN
          STD2 = STD*2
          DO J = 1,NROW
            YSQ = (REAL(J-JC))**2
            DO I = 1,NCOL
              DIST = (REAL(I-IC))**2 + YSQ 
              IF (DIST.GE.RSQ1 .AND. DIST.LE.RSQ2) THEN
                TMP    = IMG(I,J)
                IF(ABS(TMP-AVG).LT.STD2) THEN     
                  BACK   = BACK+TMP
                      TOTPTS = TOTPTS+1.0
                    SUMSQ  = SUMSQ+TMP*TMP
                  ENDIF
                ENDIF
            ENDDO
          ENDDO
          STD = 0.0
          AVG = BACK/TOTPTS
          VAR = (SUMSQ/TOTPTS) - (AVG*AVG)
          IF(VAR.GE.0.0) STD = SQRT(VAR)
        ENDIF
          NEW_BKG = NINT(AVG)                  ! NEW IMG_BKG FOR IMAGE
C  *********************************************
C  *    REPLACE OLD BACKGROUND WITH NEW_BKG    *
C  *********************************************
        DO J = 1,NROW
          YSQ = (REAL(J-JC))**2
          DO I = 1,NCOL
            DIST = (REAL(I-IC))**2 + YSQ 
            IF(DIST.LE.RSQ2) THEN
            IMG(I,J) = IMG(I,J) + IMG_BKG - NEW_BKG            ! UNFLOAT IMG
            ENDIF
            IMG(I,J) = MIN(IMG(I,J),255.0)
            IMG(I,J) = MAX(IMG(I,J),0.0)
          ENDDO
        ENDDO
C
      ELSE 
        DO J=1,NROW
          DO I=1,NCOL
            XPIX = REAL(IMG(I, J))
            CALL TRUS_PLYOUT (I, J, XPIX)
            IMG(I,J) = IMG_BKG + XPIX     
          ENDDO
        ENDDO
      ENDIF             
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
      PRINT *,' THERE WERE ',INUM,' POINTS IN BACKGROUND'
      PRINT *,' OUT OF ',REAL(NCOL)*REAL(NROW)
      PRINT *,' FLOATED BACKGROUND=',NEW_BKG
      IF(ISOL.NE.1)PRINT *,' ISOL=',ISOL
      IF(IDSOL.NE.1)PRINT *,' IDSOL=',IDSOL
       WRITE(*,'(A)') ' COEFFICIENTS: '
      WRITE(*,1000) (T(I,NNFIT),I=1,NFIT)
 1000      FORMAT(1X,E12.6,' X + ',E12.6,' Y + ',E12.6)
C
      RETURN
      END

C  ***************************************************************************
C  *                                                                         *
C  *                  *****     SUBROUTINE R4_IMG_MAP     *****                 *
C  *                                                                         *
C  *    CONVERT "IMAGE" DATA TO "MAP" DATA                                   *
C  *                                                                         *
C  *    MARCH 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                   *
C  *    FEBRUARY 1989  PURDUE UNIVERSITY 8550, T. BAKER                      *
C  *    LAST MODIFIED 15-DEC-1997  BY T. BAKER (RID OF EM.CMM).              *
C  ***************************************************************************
C
      SUBROUTINE R4_IMG_MAP (IMG, MAP, NCOL, NROW, IMG_BKG)
C
      IMPLICIT NONE
C
      INTEGER*4 ARRAY_SIZE, I, NCOL, NROW
      real*4    IMG_BKG
      real*4    IMG(1)
      REAL*4    MAP(1)
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        MAP(I) = IMG(I) - IMG_BKG
      ENDDO
C
      RETURN
      END

C  ************************************************************************
C  *                                                                      *
C  *               *****     SUBROUTINE R4_IMG_MASK      *****               *
C  *                                                                      *
C  *    REPLACE VALUES OUTSIDE CIRCULAR ANNULUS DEFINED BY IRAD1,IRAD2    *
C  *        WITH USER-DEFINED VALUE (IDENS)                               *
C  *                                                                      *
C  *    MARCH 1993  PURDUE UNIVERSITY VAX 8550, T. BAKER.                 *
C  *    LAST MODIFIED 24-MAR-1993 BY T. BAKER.                            *
C  ************************************************************************
C
      SUBROUTINE R4_IMG_MASK (IMG, NCOL, NROW, IRAD1, IRAD2, XCEN,
     1 YCEN, IDENS)
C
      IMPLICIT NONE
C
      REAL*4    RSQ1, RSQ2, XCEN, YCEN, XSQ, YSQ, TEST
      INTEGER*4 I, J, NCOL, NROW, IRAD1, IRAD2, IDENS
      real*4    IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      RSQ1 = REAL(IRAD1)**2
      RSQ2 = REAL(IRAD2)**2
C
      DO J = 1,NROW
        YSQ = (REAL(J)-YCEN)**2
        DO I = 1,NCOL
          XSQ = (REAL(I)-XCEN)**2
          TEST = XSQ + YSQ
          IF(TEST.GT.RSQ2 .OR. TEST.LT.RSQ1) IMG(I,J) = IDENS
        ENDDO
      ENDDO
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE R4_IMG_MEDIANFLT     *****           *
C  *                                                                          *
C  *    MEDIAN FILTERING ROUTINE FOR IMAGE DATA.                              *
C  *                                                                          *
C  *    NCYCLE = NUMBER OF CYCLES OF FILTERING PERFORMED.                     *
C  *             (4 IS USUALLY ENOUGH)                                        *
C  *                                                                          *
C  *    ADAPTED FROM SUBROUTINE MEDAV WRITTEN BY W. GROCHULSKI.               *
C  *                                                                          *
C  *    FEBRUARY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                   *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************

      SUBROUTINE R4_IMG_MEDIANFLT (IMG, NCOL, NROW, NCYCLE, IMG_MIN,
     $     IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE

      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, NCYCLE, I, J, K
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      INTEGER*4 C, D, E, I2_MEDIAN_3

      real*4, allocatable :: IMG2(:,:)
      INTEGER   ALLOC_STAT

      allocate(img2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_MEDIANFLT'
      endif

      DO K = 1,NCYCLE
C  *******************************
C  *    NOW FILTER ALONG ROWS    *
C  *******************************
        DO J = 1,NROW
          C = IMG(1,J)
          D = IMG(2,J)
          E = IMG(NCOL,J)
          IMG2(1,J)    = I2_MEDIAN_3 (C,D,E)
          IMG2(NCOL,J) = I2_MEDIAN_3 (C,IMG(NCOL-1,J),E)
          DO I = 2,NCOL-1
            E = IMG(I+1,J)
            IMG2(I,J) = I2_MEDIAN_3 (C,D,E)      
            C = D
            D = E
          ENDDO
        ENDDO
C  **********************************
C  *    NOW FILTER ALONG COLUMNS    *
C  **********************************
        DO I = 1,NCOL
          C           = IMG2(I,1)
          D           = IMG2(I,2)
          E           = IMG2(I,NROW)
          IMG(I,1)    = I2_MEDIAN_3 (C,D,E)
          IMG(I,NROW) = I2_MEDIAN_3 (C,IMG2(I,NROW-1),E)
          DO J = 2,NROW-1
            E        = IMG2(I,J+1)
            IMG(I,J) = I2_MEDIAN_3 (C,D,E)
            C        = D
            D        = E
          ENDDO
        ENDDO
      ENDDO

      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)

      deallocate(img2)
      RETURN
      END

C  **************************************************************
C  *                                                            *
C  *          *****     SUBROUTINE R4_IMG_MERGE     *****          *
C  *                                                            *
C  *    MERGE TWO IMAGES USING DIFFERENT BIT PLANES.            *
C  *                                                            *
C  *    JULY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.        *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM). *
C  **************************************************************
C
      SUBROUTINE R4_IMG_MERGE (IMG1, IMG2, NCOL, NROW, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG1(1), IMG2(1)
C -----------------------------------------------------------------------------
C      
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG1(I) = INT(IMG1(I)/16) + INT(IMG2(I)/16)*16
      ENDDO
C
      CALL R4_IMG_STATS (IMG2, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE R4_IMG_MULTIPLY     *****        *
C  *                                                           *
C  *    MULTIPLY ENTIRE IMAGE ARRAY BY "MULT".                 *
C  *                                                           *
C  *    DECEMBER 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER     *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).*
C  *************************************************************
C
      SUBROUTINE R4_IMG_MULTIPLY (IMG, NCOL, NROW, MULT, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, MULT
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG
      real*4    IMG(1)
C -----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG(I) = NINT(REAL(IMG(I))*MULT)
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *          *****     SUBROUTINE R4_IMG_NOISE     *****         *
C  *                                                           *
C  *    ADD RANDOM NOISE TO "IMAGE"                            *
C  *                                                           *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER     *
C  *    DECEMBER 1989  PURDUE UNIVERSITY 8550, T. BAKER        *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).*
C  *************************************************************
C
      SUBROUTINE R4_IMG_NOISE (IMG, NCOL, NROW, NOISE, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, NOISE, RAN
      INTEGER*4 NCOL, NROW, ISEED, LIM, N, NUMCHARI4
      INTEGER*4 ARRAY_SIZE, I
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(1)
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      LIM        = NINT(NOISE)
      N          = NUMCHARI4(LIM)
      WRITE(*,'(A,I10,A)') ' Add +/-', LIM, '  noise to IMAGE'
C
      CALL SEED (ISEED)
C
      DO I = 1,ARRAY_SIZE
        IMG(I) = IMG(I) + NINT((RAN(ISEED)-0.5)*2.0*NOISE)
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
C
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4_IMG_POLAR     *****                *
C  *                                                                          *
C  *    CONVERT CARTESIAN IMAGE TO POLAR IMAGE AT XCEN, YCEN CENTER           *
C  *                                                                          *
C  *    IMG1 AND IMG2 ARE THE VIRTUAL ARRAYS CONTAINING THE CARTESIAN AND     *
C  *     POLAR IMAGE DATA RESPECTIVELY                                        *
C  *                                                                          *
C  *    LIMIT ON NROT = 4096                                                  *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                                          *
C  *    JUNE 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                     *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_POLAR (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, NROT,
     1 R, NANNULI, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, D1, D2, D3, D4
      REAL*4    DX, DY, XHI, YHI, X, Y, R, DR, RSTEP
      REAL*4    XGRID(4096), YGRID(4096), IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, NANNULI, NROT
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG
      real*4    IMG1(NCOL,NROW), IMG2(NROT,NANNULI)
      INTEGER*4 I, K, IX, IY, IR
      real*8  deg_to_rad
C  ----------------------------------------------------------------------------
C
      XHI  = NCOL-1
      YHI  = NROW-1
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      deg_to_rad = acos(-1.0d0)/180.0d0

      A11 = COS(deg_to_rad*360.0/REAL(NROT))
      A12 = SIN(deg_to_rad*360.0/REAL(NROT))
      A21 = -A12
      A22 =  A11
C
      Y     = YCEN
      RSTEP = R/REAL(NANNULI)
C
      DO IR = 1,NANNULI
        DR = (IR-1)*RSTEP
        X  = XCEN+DR
        CALL R4_IMG_POLAR_GRID (X, Y, NROT, XCEN, YCEN, XGRID, YGRID)
        DO K = 1,NROT
          IF(XGRID(K).LT.1.0.OR.XGRID(K).GT.XHI.OR.
     1      YGRID(K).LT.1.0.OR.YGRID(K).GT.YHI) THEN
             IMG2(IR,K) = IMG_BKG
           ELSE
             IX = XGRID(K)
             IY = YGRID(K)
             IF(IX.GE.1.AND.IY.GE.1) THEN
               D1   = IMG1(IX  ,IY  )
               D2   = IMG1(IX+1,IY  )
               D3   = IMG1(IX+1,IY+1)
               D4   = IMG1(IX  ,IY+1)
               DX   = XGRID(K)-IX
               DY   = YGRID(K)-IY
               IMG2(K,IR) = D1+DX*(D2-D1)+DY*(D4-D1)+
     1                     DX*DY*(D1-D2+D3-D4)
             ENDIF
            ENDIF
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG2, NROT, NANNULI, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *       *****     SUBROUTINE R4_IMG_POLAR_GRID     *****       *
C  *                                                           *
C  *    FIND POSITIONS OF SYMMETRY RELATED POINTS              *
C  *    LIMIT: NROT<2049                                       *
C  *************************************************************
C
      SUBROUTINE R4_IMG_POLAR_GRID (X,Y,NROT,XCEN,YCEN,XGRID,YGRID)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      INTEGER*4 K, NROT
      REAL*4    X, Y, XCEN, YCEN, XGRID(NROT), YGRID(NROT)
      REAL*4    A11, A12, A21, A22
C  ----------------------------------------------------------------------------
C
      XGRID(1) = X
      YGRID(1) = Y
      DO K = 2,NROT
        XGRID(K) = A11*(XGRID(K-1)-XCEN)+A12*(YGRID(K-1)-YCEN)+XCEN
        YGRID(K) = A21*(XGRID(K-1)-XCEN)+A22*(YGRID(K-1)-YCEN)+YCEN
      ENDDO
C
      RETURN
      END

C  ********************************************************************
C  *                                                                  *
C  *             *****     SUBROUTINE R4_IMG_POW     *****               *
C  *                                                                  *
C  *    ROUTINE TO DETERMINE THE SUM OF IMG*IMG IN "IMAGE"            *
C  *                                                                  *
C  *    FEBRUARY 1985  T. BAKER, PURDUE UNIVERSITY VAX 11/750         *
C  *    AUGUST 1989  PURDUE UNIVERSITY 8550, T. BAKER.                *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).       *
C  ********************************************************************
C
      SUBROUTINE R4_IMG_POW (IMG, NCOL, NROW, POW, IMG_MIN, IMG_MAX,
     1   IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, J
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(1)
      REAL*4    POW
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      WRITE(*,1000)
      POW = 0.0
C
      DO I = 1,ARRAY_SIZE
        POW = POW + IMG(I)**2
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      WRITE(*,1010) POW
      RETURN
C
 1000      FORMAT(' Compute IMAGE power: Wait for answer')
 1010      FORMAT('+Compute IMAGE power:',F12.0,'    ')
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4_IMG_ROT     *****                  *
C  *                                                                          *
C  *    ROTATE IMAGE ABOUT ITS CENTER BY "RANGL" DEGREES                      *
C  *                                                                          *
C  *    IMG1 AND IMG2 ARE THE VIRTUAL ARRAYS CONTAINING THE ORIGINAL AND      *
C  *     ROTATED IMAGE DATA RESPECTIVELY                                      *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                             *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_ROT (IMG1, IMG2, NCOL, NROW, NCOL2, NROW2,
     1     RANGL, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    XCEN1, YCEN1, XCEN2, YCEN2, A11, A12, A21, A22
      REAL*4    D1, D2, D3, D4, DX, DY, X, Y, RANGL, IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, NCOL2, NROW2, IX, IY, I, J
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG1(NCOL,NROW), IMG2(NCOL2,NROW2)
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
      XCEN1 = REAL(NCOL+1)/2.0
      YCEN1 = REAL(NROW+1)/2.0
      XCEN2 = REAL(NCOL2+1)/2.0
      YCEN2 = REAL(NROW2+1)/2.0
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      deg_to_rad = acos(-1.0d0)/180.0d0
      A11 = COS(RANGL*deg_to_rad)
      A12 = SIN(RANGL*deg_to_rad)
      A21 = -A12
      A22 =  A11
C
      DO J = 1,NROW2                  ! STEP THROUGH ROWS OF IMG2
        DO I = 1,NCOL2            ! STEP THROUGH CURRENT ROW OF IMG2
          X = A11*(REAL(I)-XCEN2)+A12*(REAL(J)-YCEN2)+XCEN1
          Y = A21*(REAL(I)-XCEN2)+A22*(REAL(J)-YCEN2)+YCEN1
          IX = X
          IY = Y
          IF(IX.GE.1.AND.IY.GE.1.AND.IX.LE.NCOL.AND.IY.LE.NROW) THEN
             D2        = IMG_BKG
             D3        = IMG_BKG
             D4        = IMG_BKG
             D1        = IMG1(IX,IY)
             IF(IX.LT.NCOL) D2 = IMG1(IX+1,IY)
             IF(IX.LT.NCOL.AND.IY.LT.NROW) D3 = IMG1(IX+1,IY+1)
             IF(IY.LT.NROW) D4 = IMG1(IX,IY+1)
             DX        = X-IX
             DY        = Y-IY
             IMG2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
           ELSE
             IMG2(I,J) = IMG_BKG
          ENDIF
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG2, NCOL2, NROW2, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *              *****     SUBROUTINE R4_IMG_SCALE     *****             *
C  *                                                                   *
C  *    ROUTINE TO RESCALE "IMAGE" VALUES FROM INT1_OLD-INT2_OLD TO    *
C  *     INT1_NEW-INT2_NEW                                             *
C  *                                                                   *
C  *    NOTE THAT SCALING IS ONLY DONE WITH NON-ZERO VALUES, I.E.      *
C  *     INTENSITY VALUES THAT = 0 ARE IGNORED BECAUSE THEY ARE        *
C  *     ASSUMED TO BE BACKGROUND VALUES.                              *
C  *                                                                   *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER             *
C  *    AUGUST 1987  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).        *
C  *********************************************************************
C
      SUBROUTINE R4_IMG_SCALE (IMG, NCOL, NROW, INT1_OLD, INT2_OLD,
     1 INT1_NEW, INT2_NEW, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, SCALE
      INTEGER*4 NCOL, NROW, INT1_OLD, INT2_OLD, INT1_NEW, INT2_NEW
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, img(1)
      INTEGER*4 I, N1, N2, N3, N4, NUMCHARI4, ARRAY_SIZE
      INTEGER*4 ITMP
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      N1         = NUMCHARI4(INT1_OLD)
      N2         = NUMCHARI4(INT2_OLD)
      N3         = NUMCHARI4(INT1_NEW)
      N4         = NUMCHARI4(INT2_NEW)
C
      WRITE(*,1000) INT1_OLD, INT2_OLD, INT1_NEW, INT2_NEW
C
      SCALE = 1.0
      IF(INT1_OLD.NE.INT2_OLD) THEN
        SCALE = REAL(INT2_NEW-INT1_NEW)/REAL(INT2_OLD-INT1_OLD)
      ENDIF
C
      IF(INT1_OLD.LT.0) THEN
         DO I = 1,ARRAY_SIZE
           ITMP = IMG(I)
           IF(ITMP.GE.INT1_OLD.AND.ITMP.LE.INT2_OLD) THEN
             IMG(I) = INT1_NEW + (ITMP-INT1_OLD)*SCALE
           ENDIF
         ENDDO
       ELSE
         DO I = 1,ARRAY_SIZE
           ITMP = IMG(I)
           IF(ITMP.NE.0) THEN
              IF(ITMP.GE.INT1_OLD.AND.ITMP.LE.INT2_OLD) THEN
                IMG(I) = INT1_NEW + (ITMP-INT1_OLD)*SCALE
              ENDIF
            ELSE
              IF(INT1_OLD.EQ.0) IMG(I) = INT1_NEW
           ENDIF
         ENDDO
      ENDIF
      IMG_BKG = INT1_NEW + (IMG_BKG-INT1_OLD)*SCALE
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
      RETURN
C
 1000      FORMAT(' Rescale image values between ',I10,' and ',I10,
     1      ' from ',I10,' to ',I10)
      END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE R4_IMG_SHIFT     *****              *
C  *                                                                          *
C  *    SHIFT DATA WITHIN IMG BY XSHIFT,YSHIFT.                               *
C  *                                                                          *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                     *
C  *                                                                          *
C  *          D4            D3                                                *
C  *                                                                          *
C  *                    + (dx,dy)                                             *
C  *                    .                                                     *
C  *          D1. . . . .   D2                                                *
C  *                                                             *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                   *
C  *                                                                          *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)       *
C  *                                                                          *
C  *    WHICH REDUCES TO:                                                     *
C  *                                                                          *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                  *
C  *                                                                          *
C  *    FEBRUARY 1989 PURDUE UNIVERSITY VAX 8550, T. BAKER                    *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C

      SUBROUTINE R4_IMG_SHIFT (IMG, NCOL, NROW, XSHIFT, YSHIFT,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      REAL*4    X, Y, DX, DY, XSHIFT, YSHIFT, D1, D2, D3, D4
      INTEGER*4 NCOL, NROW, I, J, IX1, IX2, IY1, IY2
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      LOGICAL   XSTART, XEND, YSTART, YEND

      real*4, allocatable :: IMG2(:,:)
      INTEGER   ALLOC_STAT

      allocate(img2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_SHIFT'
      endif

      DO J = 1,NROW
        Y   = J - YSHIFT
        IY1 = INT(Y)
        IY2 = IY1 + 1
        DY  = Y - IY1
        IF(IY1.GE.1 .AND. IY1.LE.NROW) THEN
           YSTART = .TRUE.
         ELSE
           YSTART = .FALSE.
        ENDIF
        IF(IY2.GE.1 .AND. IY2.LE.NROW) THEN
           YEND = .TRUE.
         ELSE
           YEND = .FALSE.
        ENDIF
        DO I = 1,NCOL
          X   = I - XSHIFT
          IX1 = INT(X)
          IX2 = IX1 + 1
          DX  = X - IX1
          IF(IX1.GE.1 .AND. IX1.LE.NCOL) THEN
             XSTART = .TRUE.
           ELSE
             XSTART = .FALSE.
          ENDIF
          IF(IX2.GE.1 .AND. IX2.LE.NCOL) THEN
             XEND = .TRUE.
           ELSE
             XEND = .FALSE.
          ENDIF
          IF(XSTART.AND.YSTART) THEN
             D1 = IMG(IX1,IY1)
           ELSE
             D1 = IMG_BKG
          ENDIF
          IF(XEND.AND.YSTART) THEN
             D2 = IMG(IX2,IY1)
           ELSE
             D2 = IMG_BKG
          ENDIF
          IF(XEND.AND.YEND) THEN
             D3 = IMG(IX2,IY2)
           ELSE
             D3 = IMG_BKG
          ENDIF
          IF(XSTART.AND.YEND) THEN
             D4 = IMG(IX1,IY2)
           ELSE
             D4 = IMG_BKG
          ENDIF

          IMG2(I,J) = D1 + (D2-D1)*DX + (D4-D1)*DY
     1                  + (D1-D2+D3-D4)*DX*DY

        ENDDO
      ENDDO
C  ********************************
C  *    COPY "IMG2" INTO "IMG"    *
C  ********************************
      img(1:ncol,1:nrow) = img2(1:ncol,1:nrow) ! Fortran 90 syntax

      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      deallocate(img2)

      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE R4_IMG_STATS     *****            *
C  *                                                                 *
C  *    DETERMINE IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD     *
C  *     OF "IMAGE" VALUES.                                          *
C  *                                                                 *
C  *    NOTE THAT THE VALUE DETERMINED FOR IMG_MIN IS THE MINIMUM    *
C  *     NON-ZERO VALUE IN THE IMG ARRAY                             *
C  *                                                                 *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).      *
C  *******************************************************************
C
      SUBROUTINE R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX,
     1    IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      REAL*4    IMG_AVG, IMG_STD, TOT, SUMSQ, VAR, X
      real*4    IMG_MIN, IMG_MAX
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, J
      real*4    IMG(1), IMG_BKG
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
Ca      IMG_MIN    = I2_MAX
Ca      IMG_MAX    = I2_MIN
      img_min = img(1)
      img_max = img(1)
      SUMSQ      =    0.0
      TOT        =    0.0
C
      WRITE(*,1000) 
C
      DO I = 1,ARRAY_SIZE
        X          = IMG(I)
        SUMSQ      = SUMSQ + X*X
        TOT        = TOT + X
        IF(IMG(I).NE.0) THEN
Ca          IMG_MIN = MIN0(IMG_MIN,IMG(I))
Ca          IMG_MAX = MAX0(IMG_MAX,IMG(I))
          IMG_MIN = MIN(IMG_MIN,IMG(I))
          IMG_MAX = MAX(IMG_MAX,IMG(I))
        ENDIF
      ENDDO
C
      IMG_AVG = TOT/REAL(ARRAY_SIZE)
      IMG_STD = 0.0
      VAR     = (SUMSQ/REAL(ARRAY_SIZE)) - IMG_AVG**2
      IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
C  *************************
C  *    COMPUTE IMG_BKG    *
C  *************************
      TOT = 0.0
      DO I = 1,NCOL                                    ! BOTTOM ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = ARRAY_SIZE-NCOL+1,ARRAY_SIZE                  ! TOP ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = NCOL+1,ARRAY_SIZE-NCOL,NCOL                  ! LEFT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      DO I = 2*NCOL,ARRAY_SIZE-NCOL,NCOL                  ! RIGHT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      IMG_BKG = TOT/(2*NCOL+2*(NROW-2))
C
      WRITE(*,1010) IMG_MIN,IMG_MAX,IMG_AVG,IMG_STD,IMG_BKG
      RETURN
C
 1000      FORMAT(
     1 /,' *****************************************************',
     2 /,' *   NON-ZERO                                        *',
     3 /,' *   MINIMUM   MAXIMUM   AVERAGE        STD     BKG  *',
     4 /,' * ------------------------------------------------- *',
     5 /,' *            ...  WAIT for answer  ...              *')
 1010      FORMAT('+*',f8.1,f10.1,2F12.2,f7.1,'  *',
     1 /,' *****************************************************',/)
C
      END


C  *******************************************************************
C  *                                                                 *
C  *           *****     SUBROUTINE R4_IMG_STATS_FLAG     *****         *
C  *                                                                 *
C  *    DETERMINE IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD     *
C  *     OF "IMAGE" VALUES WHILE IGNORING ANY VALUES = FLAG.         *
C  *    NOTE: ROUTINE ALWAYS TRAPS FOR A VALUE OF 255.               *
C  *                                                                 *
C  *    JULY 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).      *
C  *******************************************************************
C
      SUBROUTINE R4_IMG_STATS_FLAG (IMG, NCOL, NROW, FLAG, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      REAL*4    IMG_AVG, IMG_STD, TOT, SUMSQ, VAR, X
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, J, COUNTER
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(1)
      INTEGER*4 FLAG
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      IMG_MIN    = I2_MAX
      IMG_MAX    = I2_MIN
      img_min  = img(1)
      img_max  = img(1)
      SUMSQ      =    0.0
      TOT        =    0.0
      COUNTER    =      0
C
      WRITE(*,1000) FLAG
C
      IF(FLAG.NE.255) THEN
         DO I = 1,ARRAY_SIZE
           IF(IMG(I).NE.FLAG .AND. IMG(I).NE.255) THEN
             COUNTER = COUNTER + 1
             X       = REAL(IMG(I))
             SUMSQ   = SUMSQ + X*X
             TOT     = TOT + X
Ca             IMG_MIN = MIN0(IMG_MIN,IMG(I))
Ca             IMG_MAX = MAX0(IMG_MAX,IMG(I))
             IMG_MIN = MIN(IMG_MIN,IMG(I))
             IMG_MAX = MAX(IMG_MAX,IMG(I))
           ENDIF
         ENDDO
       ELSE
         DO I = 1,ARRAY_SIZE
           IF(IMG(I).NE.FLAG) THEN
             COUNTER = COUNTER + 1
             X       = REAL(IMG(I))
             SUMSQ   = SUMSQ + X*X
             TOT     = TOT + X
Ca             IMG_MIN = MIN0(IMG_MIN,IMG(I))
Ca             IMG_MAX = MAX0(IMG_MAX,IMG(I))
             IMG_MIN = MIN(IMG_MIN,IMG(I))
             IMG_MAX = MAX(IMG_MAX,IMG(I))
           ENDIF
         ENDDO
      ENDIF
C
      IMG_AVG = TOT/REAL(COUNTER)
      IMG_STD = 0.0
      VAR     = (SUMSQ/REAL(COUNTER)) - IMG_AVG**2
      IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
C  *************************
C  *    COMPUTE IMG_BKG    *
C  *************************
      TOT = 0.0
      DO I = 1,NCOL                                    ! BOTTOM ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = ARRAY_SIZE-NCOL+1,ARRAY_SIZE                  ! TOP ROW
        TOT = TOT + IMG(I)
      ENDDO
      DO I = NCOL+1,ARRAY_SIZE-NCOL,NCOL                  ! LEFT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      DO I = 2*NCOL,ARRAY_SIZE-NCOL,NCOL                  ! RIGHT COLUMN
        TOT = TOT + IMG(I)
      ENDDO
      IMG_BKG = TOT/(2*NCOL+2*(NROW-2))
C
      WRITE(*,1010) IMG_MIN,IMG_MAX,IMG_AVG,IMG_STD,IMG_BKG
      RETURN
C
 1000      FORMAT(
     1 /,' *****************************************************',
     2 /,' *   IMG STATS (IGNORES A FLAG VALUE OF ',I3,')         *',
     3 /,' *   MINIMUM   MAXIMUM   AVERAGE        STD     BKG  *',
     4 /,' * ------------------------------------------------- *',
     5 /,' *            ...  WAIT for answer  ...              *')
 1010      FORMAT('+*',I8,I10,2F12.2,I7,'  *',
     1 /,' *****************************************************',/)
      END


C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE R4_IMG_STATS_NN  *****            *
C  *                                                                 *
C  *    DETERMINE IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD OF "IMAGE"   *
C  *     VALUES                                                      *
C  *                                                                 *
C  *    NOTE THAT THE VALUE DETERMINED FOR IMG_MIN IS THE MINIMUM    *
C  *     NON-ZERO VALUE IN THE IMG ARRAY                             *
C  *                                                                 *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    MODIFIED AUG 1990  BY NRD@BRAGG.BIO.PURDUE.EDU                   *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).      *
C  *******************************************************************
C
      SUBROUTINE R4_IMG_STATS_NN (IMG, NCOL, NROW, FOCUS_FACTOR, 
     $     IMG_STD)
C     
      IMPLICIT NONE
C
      INTEGER*4 COUNTER, NCOL, NROW, I, J, IMG_STD
      real*4    IMG(NCOL,NROW)
      REAL*4    FOCUS_SUMSQ, FOCUS_STD, FOCUS_VAR, FOCUS_FACTOR
      REAL*4        FOCUS_TOT, FOCUS_STD_TOT, FOCUS_AVG_STD 
C  ----------------------------------------------------------------------------
C
      FOCUS_STD_TOT = 0.0
      COUNTER       =   0
C
      DO I = 2,NCOL-1
        DO J = 2,NROW-1
          IF (IMG(I,J).NE.255 .AND. IMG(I+1,J).NE.255 .AND. 
     1     IMG(I-1,J).NE.255 .AND. IMG(I,J+1).NE.255 .AND. 
     2     IMG(I,J-1).NE.255) THEN
            FOCUS_SUMSQ = REAL(IMG(I,J))**2 + REAL(IMG(I+1,J))**2 + 
     1      REAL(IMG(I-1,J))**2 + REAL(IMG(I,J+1))**2 + 
     2      REAL(IMG(I,J-1))**2
            FOCUS_TOT = REAL(IMG(I,J)) + REAL(IMG(I+1,J)) + 
     1      REAL(IMG(I-1,J)) + REAL(IMG(I,J+1)) + REAL(IMG(I,J-1))
            FOCUS_VAR = FOCUS_SUMSQ/5.0-(FOCUS_TOT/5.0)**2
            IF (FOCUS_VAR.GE.0.0) THEN
            FOCUS_STD     = SQRT(FOCUS_VAR)
              FOCUS_STD_TOT = FOCUS_STD_TOT + FOCUS_STD
            COUNTER       = COUNTER + 1
            ENDIF
          ENDIF
        ENDDO
      ENDDO
C
      FOCUS_AVG_STD = FOCUS_STD_TOT/REAL(COUNTER)
      FOCUS_FACTOR  = FOCUS_AVG_STD/IMG_STD
      IF (IMG_STD.NE.0.0) THEN 
        WRITE (6,1000) FOCUS_AVG_STD, FOCUS_FACTOR 
      ENDIF
C
      RETURN
C
 1000   FORMAT(
     1 /,' **********************************************',
     1 /,' *            IMAGE FOCUS STATISTICS          *',
     2 /,' *--------------------------------------------*',
     2 /,' *   AVERAGE NEAREST-NEIGHBOR STD: ',F8.2  ,'   *',
     2 /,' *             DIVIDED BY IMG_STD: ',F8.2  ,'   *',
     3 /,' **********************************************',/)
C      
      END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE R4_IMG_SYM     *****                  *
C  *                                                                          *
C  *    APPLY NROT ROTATIONAL SYMMETRY TO IMAGE AT XCEN, YCEN                 *
C  *                                                                          *
C  *    IMG1 AND IMG2 ARE THE VIRTUAL ARRAYS CONTAINING THE ORIGINAL AND      *
C  *     SYMMETRIZED IMAGE DATA RESPECTIVELY                                  *
C  *                                                                          *
C  *    LIMIT ON NROT = 100, IF NROT>100 THEN COMPUTE CIRCULAR AVERAGE        *
C  *                                                                          *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_SYM (IMG1, NCOL, NROW, XCEN, YCEN, NROT,
     1     IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG, rint, numAlloc)
C
      IMPLICIT NONE
C
      INTEGER*4 NROT, NCOL, NROW, I, numAlloc
      real*4    IMG_MIN, IMG_MAX, IMG_BKG
      real*4    IMG1(1)
      real*4, allocatable :: IMG2(:)
      REAL*4    IMG_AVG, IMG_STD, XCEN, YCEN, rint(numAlloc)
      INTEGER   ALLOC_STAT
C  ----------------------------------------------------------------------------

      allocate(img2(ncol*nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_AVERAGEFLT'
      endif

      IF(NROT.GT.100) THEN
         CALL R4_IMG_SYM_CAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, NROT,
     1      IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG, rint, numAlloc)
       ELSE
         CALL R4_IMG_SYM_RAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, NROT,
     1      IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG, rint)
      ENDIF
C
      img1(1:ncol*nrow) = img2(1:ncol*nrow) ! Fortran 90 syntax
      deallocate(img2)
Ca      CALL R4_IMG_STATS (IMG1, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
Ca     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE R4_IMG_SYM_CAVG     *****        *
C  *                                                           *
C  *    NROT > 100: COMPUTE CIRCULARLY AVERAGED IMAGE          *
C  *************************************************************
C
      SUBROUTINE R4_IMG_SYM_CAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN, 
     $     NROT, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG, 
     $     rint, numAlloc)
C
      IMPLICIT NONE
C
      INTEGER*4 NROT, NCOL, NROW, numAlloc
      real*4    IMG_MIN, IMG_MAX, IMG_BKG
      INTEGER*4 INUM, I, J, K, IR
      real*4    INTERP
      real*4    IRNUM(10000)
      real*4    IMG1(NCOL,NROW), IMG2(NCOL,NROW)
      REAL*4    IMG_AVG, IMG_STD, XCEN, YCEN, X, Y, YSQ, rint(numAlloc)
      PARAMETER (INTERP=2)
C  ----------------------------------------------------------------------------
C
      INUM = NINT(INTERP*SQRT(REAL(NROW)**2+REAL(NCOL)**2))
      DO I = 1,INUM            ! INITIALIZE 1D INTERPOLATION ARRAY
        RINT(I)  = 0.0
        IRNUM(I) = 0.0
      ENDDO
C  *************************************************************
C  *    STEP THROUGH IMAGE AND REDUCE TO 1D CIRCULAR AVERAGE   *
C  *************************************************************
      DO J = 1,NROW
        Y   = REAL(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = REAL(I)-XCEN
          IR        = nint(INTERP*SQRT(X*X+YSQ))+1
          IRNUM(IR) = IRNUM(IR)+1.0
          RINT(IR)  = RINT(IR)+IMG1(I,J)
        ENDDO
      ENDDO
C  ********************************
C  *    NORMALIZE THE 1D ARRAY    *
C  ********************************
      DO I = 1,INUM
        IF(IRNUM(I).GT.1.0) RINT(I) = RINT(I)/IRNUM(I)
      ENDDO
C  **************************************************
C  *    FILL IMG2 WITH CIRCULARLY AVERAGED IMAGE    *
C  **************************************************
      DO J = 1,NROW
        Y   = REAL(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X         = REAL(I)-XCEN
          IR        = nint(INTERP*SQRT(X*X+YSQ))+1
          IMG2(I,J) = RINT(IR)
        ENDDO
      ENDDO
C
Ca      CALL R4_IMG_STATS (IMG2, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
Ca     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *        *****     SUBROUTINE R4_IMG_SYM_RAVG     *****          *
C  *                                                             *
C  *    NROT < 101: COMPUTE ROTATIONAL AVERAGE OF IMAGE          *
C  *                                                             *
C  *    LAST MODIFIED 27-JUN-1991 BY T. BAKER.                   *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).  *
C  ***************************************************************
C
      SUBROUTINE R4_IMG_SYM_RAVG (IMG1, IMG2, NCOL, NROW, XCEN, YCEN,
     1   NROT, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      REAL*4    IMG_AVG, IMG_STD
      REAL*4    XCEN, YCEN, A11, A12, A21, A22, DENS, D1, D2, D3, D4
      REAL*4    DX, DY, TOT, XGRID(100), YGRID(100), X, Y
      INTEGER*4 NROT, NCOL, NROW, I, J, K
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG,IMG1(NCOL,NROW), IMG2(NCOL,NROW)
      real*8    deg_to_rad
      INTEGER*4 IX, IY, IOUT, INUM, NUM, IR
      INTEGER*4 NGRID, ITST, ILST
      DATA IOUT/-9999/
C  ----------------------------------------------------------------------------
C
C  ********************************
C  *    SET UP ROTATION MATRIX    *
C  ********************************
      deg_to_rad = acos(-1.0d0)/180.0d0
      A11 = COS(deg_to_rad*360.0/REAL(NROT))
      A12 = SIN(deg_to_rad*360.0/REAL(NROT))
      A21 = -A12
      A22 =  A11
C
      DO J = 1,NCOL                  ! STEP THROUGH COLUMNS OF IMG1
        DO I = 1,NROW                  ! STEP THROUGH CURRENT COLUMN OF IMG1
          CALL R4_IMG_SYM_GRID (I, J, NROT, XCEN, YCEN, XGRID, YGRID)
C           **********************************************************
C           *   CHECK NUMBER OF GRID POINTS WITHIN IMAGE BOUNDARY    *
C           **********************************************************
          NGRID = 1
          DO K = 2,NROT
            IF(XGRID(K).GE.1.0.AND.XGRID(K).LE.NCOL.AND.
     1        YGRID(K).GE.1.0.AND.YGRID(K).LE.NROW) THEN
               NGRID = NGRID+1
             ELSE
               XGRID(K) = -1.0
               YGRID(K) = -1.0
            ENDIF
          ENDDO
          IF(NGRID.EQ.1) THEN
             IMG2(J,I) = IOUT
           ELSE
             DENS  = IMG1(J,I)
             NGRID = 1
             DO K = 2,NROT
               IX = XGRID(K)
               IY = YGRID(K)
               IF(IX.GE.1.AND.IY.GE.1) THEN
                 NGRID = NGRID+1
                 D2   = IMG_BKG
                 D3   = IMG_BKG
                 D4   = IMG_BKG
                 D1   = IMG1(IX,IY)
                 IF(IX.LT.NCOL) D2 = IMG1(IX+1,IY)
                 IF(IY.LT.NROW.AND.IX.LT.NCOL) D3 = IMG1(IX+1,IY+1)
                 IF(IY.LT.NROW) D4 = IMG1(IX,IY+1)
                 DX   = XGRID(K)-IX
                 DY   = YGRID(K)-IY
                 DENS = DENS+D1+DX*(D2-D1)+DY*(D4-D1)+
     1                      DX*DY*(D1-D2+D3-D4)
               ENDIF
               ENDDO
             IMG2(J,I) = NINT(DENS/REAL(NGRID))
          ENDIF
        ENDDO
      ENDDO
C  *********************************************************
C  *    COMPUTE BACKGROUND VALUE FROM POINTS ON BOUNDARY   *
C  *********************************************************
      PRINT *, '*******************************************'
      PRINT *, '* Compute BACKGROUND of symmetrized image *'
      PRINT *, '*******************************************'
      TOT  = 0.0
      NUM  =   0
      DO J = 1,NROW                              ! STEP THROUGH ROW BY ROW
        ITST = -1                                   ! OUTSIDE BOUNDARY
        ILST =  0
        DO I = 1,NCOL                                 ! STEP THROUGH ROW J
          IF(IMG2(I,J).EQ.IOUT) THEN                       ! OUTSIDE BOUNDARY
             IF(ITST.EQ.1.AND.I.NE.ILST) THEN            ! JUST LEAVING SYM AREA
               NUM = NUM+1
               TOT = TOT+IMG1(I,J)                  ! PIXEL AT RIGHT BOUNDARY
               ITST = -1                         ! NOW OUTSIDE BOUNDARY
               GO TO 100                         ! STEP TO NEXT ROW (J+1)
             ENDIF
             ITST = -1                             ! OUTSIDE BOUNDARY
           ELSE                                    ! INSIDE BOUNDARY
             IF(ITST.EQ.-1.AND.I.GT.1) THEN           ! LEAVING OUTSIDE/ENTERING
               NUM  = NUM+1                       !         INSIDE
               ILST = I+1
               TOT  = TOT+IMG1(I-1,J)        ! PREVIOUS PIXEL WAS ON LEFT BOUNDARY
             ENDIF
             ITST = 1                                    ! INSIDE BOUNDARY
          ENDIF
        ENDDO
        IF(ILST.EQ.0) THEN
C           **********************************************
C           *    GET HERE ONLY WHEN ENTIRE ROW IS ALL    *
C           *     INSIDE OR ALL OUTSIDE THE BOUNDARY     *
C           **********************************************
          IF(ITST.EQ.1) THEN                     ! ENTIRE ROW INSIDE BOUNDARY
            IF(NUM.EQ.0) THEN                   ! FIRST ROW INSIDE BOUNDARY USES
               DO I = 1,NCOL                   !      ENTIRE ROW FOR BACKGROUND
                 NUM = NUM + 1
                 TOT = TOT + IMG1(I,J)
               ENDDO
             ELSE                         ! USE EDGE VALUES FOR BACKGROUND
               NUM = NUM + 2            
               TOT = TOT + IMG1(1,J) + IMG1(NCOL,J)
            ENDIF
          ENDIF
          IF(ITST.EQ.-1) THEN                    ! ENTIRE ROW OUTSIDE BOUNDARY
            IF(NUM.NE.0) THEN                             ! THIS IS LAST ROW
              DO I = 1,NCOL
                NUM = NUM + 1
                TOT = TOT + IMG1(I,J-1)
              ENDDO
            ENDIF
          ENDIF
        ENDIF
  100      ENDDO
      IMG_BKG = TOT/REAL(NUM)
C  ************************************************
C  *    REPLACE ALL VALUES = IOUT WITH IMG_BKG    *
C  ************************************************
      DO J = 1,NROW
        DO I = 1,NCOL
          IF(IMG2(I,J).EQ.IOUT) IMG2(I,J) = IMG_BKG
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG2, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *************************************************************
C  *                                                           *
C  *        *****     SUBROUTINE R4_IMG_SYM_GRID     *****        *
C  *                                                           *
C  *    FIND POSITIONS OF SYMMETRY RELATED POINTS              *
C  *    LIMIT: NROT<101                                        *
C  *************************************************************
C
      SUBROUTINE R4_IMG_SYM_GRID (I,J,NROT,XCEN,YCEN,XGRID,YGRID)
C
      IMPLICIT NONE
C
      COMMON /ROT_MATRIX/ A11, A12, A21, A22
      REAL*4    XCEN, YCEN, XGRID(100), YGRID(100), A11, A12, A21, A22
      INTEGER*4 I, J, K, NROT
C  ----------------------------------------------------------------------------
C
      XGRID(1) = J                              ! CURRENT COLUMN POSITION
      YGRID(1) = I                              ! CURRENT ROW POSITION
      DO K = 2,NROT
        XGRID(K) = A11*(XGRID(K-1)-XCEN)+A12*(YGRID(K-1)-YCEN)+XCEN
        YGRID(K) = A21*(XGRID(K-1)-XCEN)+A22*(YGRID(K-1)-YCEN)+YCEN
      ENDDO
C
      RETURN
      END

C  **************************************************************************
C  *                                                                        *
C  *                  *****     SUBROUTINE R4_IMG_TRP     *****                *
C  *                                                                        *
C  *    INTERPOLATES AN NCOL BY NROW IMAGE PRODUCING AN NCOL2 BY NROW2      *
C  *     IMAGE                                                              *
C  *                                                                        *
C  *    POINT LAYOUT FOR BI-LINEAR INTERPOLATION SCHEME :                   *
C  *                                                                        *
C  *          D4            D3                                              *
C  *                                                                        *
C  *                    + (dx,dy)                                           *
C  *                    .                                                   *
C  *          D1. . . . .   D2                                              *
C  *                                                           *
C  *    THE INTERPOLATION ALGORITHM IS COMPUTED AS FOLLOWS:                 *
C  *                                                                        *
C  *          D1(1-dX)(1-dY) + D2(dX)(1-dY) + D3(dX)(dY) + D4(1-dX)(dY)     *
C  *                                                                        *
C  *    WHICH REDUCES TO:                                                   *
C  *                                                                        *
C  *          D1 + (D2-D1)dx + (D4-D1)dy + (D1-D2+D3-D4)dxdy                *
C  *                                                           *
C  *    AUGUST 1984  PURDUE UNIVERSITY, VAX 11/750, BOB ZIGON               *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).             *
C  **************************************************************************
C
      SUBROUTINE R4_IMG_TRP (IMG1, IMG2, NCOL, NROW, NCOL2, NROW2,
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      REAL*4    XSTEP, YSTEP, Y, X1, D1, D2, D3, D4, DIV, DX, DY
      INTEGER*4 MSTEP, NCOL, NROW, NCOL2, NROW2, IX10, I, J, IY, IX1
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG1(NCOL,NROW), IMG2(NCOL2,NROW2)
      INTEGER*4 INDX1, INDX2, INDY1, INDY2
C  ----------------------------------------------------------------------------
C
      PRINT *, '*****************************************'
      PRINT *, '* Wait while interpolation is performed *'
      PRINT *, '*****************************************'
      IX10  = 10*NCOL
      XSTEP = (REAL(NCOL)-1.0)/(REAL(NCOL2)-1.0)
      YSTEP = (REAL(NROW)-1.0)/(REAL(NROW2)-1.0)
      MSTEP = NROW2/10
      IF(MSTEP.LE.0) MSTEP = 1
      DIV   = 100.0/REAL(NROW2)
C
      DO J = 1,NROW2
         Y    = (J-1)*YSTEP
        IY    = Y
        DY    = Y-IY
        INDY1 = IY+1
        INDY2 = 1+MOD(INDY1,NROW)
        DO I = 1,NCOL2
          X1        = (I-1)*XSTEP
          IX1       = X1+IX10
          INDX1     = 1+MOD(IX1,NCOL)
          INDX2     = 1+MOD(INDX1,NCOL)
          D1        = IMG1(INDX1,INDY1)
          D2        = IMG1(INDX2,INDY1)
          D3        = IMG1(INDX2,INDY2)
          D4        = IMG1(INDX1,INDY2)
          IX1       = IX1-IX10
          DX        = X1-IX1
          IMG2(I,J) = D1+DX*(D2-D1)+DY*(D4-D1)+DX*DY*(D1-D2+D3-D4)
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG2, NCOL2, NROW2, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
       RETURN
       END

C  *********************************************************************
C  *                                                                   *
C  *             *****     SUBROUTINE R4_IMG_TRUNCATE  *****              *
C  *                                                       *
C  *            truncates intensities in a image to int1 (low) and     *
C  *      int2 (high) limits.                                      *
C  *                                                       *
C  *      Created:  July 23rd, 1990  by nrd@bragg.bio.purdue.edu         *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).        *
C  *********************************************************************
C
      SUBROUTINE R4_IMG_TRUNCATE (IMG, NCOL, NROW, INT1, INT2, IMG_MIN,
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW
      real*4    IMG_MIN, IMG_MAX, INT1, INT2
      INTEGER*4 I, ARRAY_SIZE
      real*4    IMG(1), IMG_BKG
c  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
C
      DO I = 1,ARRAY_SIZE
        IMG(I) = MIN(IMG(I),INT2)
        IMG(I) = MAX(IMG(I),INT1)
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG, 
     1   IMG_STD, IMG_BKG)
      RETURN

      END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE R4_IMG_TST_CIR     *****                *
C  *                                                                          *
C  *    ADDS OR OVERWRITES FILLED/UNFILLED CIRCLE/ANNULUS TO MODEL IMAGE      *
C  *                                                                          *
C  *    FEBRUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).               *
C  ****************************************************************************
C
      SUBROUTINE R4_IMG_TST_CIR (IMG, NCOL, NROW, XCEN, YCEN, R1, R2,
     1 IADD, INT, FILL, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    X, XCEN, XSQ, Y, YCEN, YSQ, R1, R2, RSQMIN, RSQMAX
      REAL*4    TST, YCEN2, IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, IADD, INT, I, J, FILL
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      IF(IADD.EQ.1) THEN
         IADD = 0                  ! OVERWRITE OLD IMAGE
       ELSE
         IADD = 1                  ! ADD TO OLD IMAGE
      ENDIF
C
      RSQMIN = R1*R1
      RSQMAX = R2*R2
C
      DO J = 1,NROW
        Y   = REAL(J)-YCEN
        YSQ = Y*Y
        DO I = 1,NCOL
          X   = REAL(I)-XCEN
          XSQ = X*X
          TST = XSQ+YSQ
          IF(FILL.EQ.1) THEN
            IF(TST.LE.RSQMAX.AND.TST.GE.RSQMIN) THEN
              IMG(I,J) = IADD*IMG(I,J)+INT
            ENDIF
          ELSE 
            IF ((TST.GT.(RSQMAX-R2).AND.TST.LT.(RSQMAX+R2)).OR.
     1      (TST.GT.(RSQMIN-R1).AND.TST.LT.(RSQMIN+R1))) THEN
               IMG(I,J) = IADD*IMG(I,J) + INT
            ENDIF
          ENDIF
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  ***************************************************************
C  *                                                             *
C  *         *****     SUBROUTINE R4_IMG_TST_LIN     *****          *
C  *                                                             *
C  *    ADDS OR OVERWRITES SOLID LINE TO MODEL IMAGE             *
C  *                                                             *
C  *    AUGUST 1986 PURDUE UNIVERSITY VAX 11/750, L. MATTERN     *
C  *    MODIFIED 20-OCT-86 BY L. MATTERN                         *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).  *
C  ***************************************************************
C
      SUBROUTINE R4_IMG_TST_LIN (IMG, NCOL, NROW, X1, X2, Y1, Y2,
     1 IADD, INT, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, X1, X2, Y1, Y2, X, Y, M, B, YPRIME
      INTEGER*4 NCOL, NROW, I, J, INT, IADD
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
      IF (IADD .EQ. 1) THEN
         IADD = 0
       ELSE
         IADD = 1
      END IF
C
C  *********************************
C  *    CHECK FOR SPECIAL CASES    *
C  *********************************
      IF(X1.EQ.X2) THEN                  ! VERTICAL LINE
         I = NINT(X1)
         DO J = NINT(Y1),NINT(Y2)
           IMG(I,J) = IADD * IMG(I,J) + INT
         ENDDO
         RETURN
       ELSE IF (Y1.EQ.Y2) THEN            ! HORIZONTAL LINE
         J = NINT(Y1)
         DO I = NINT(X1),NINT(X2)
           IMG(I,J) = IADD * IMG(I,J) + INT
         ENDDO
         RETURN
      ENDIF
C
      IF (X1 .NE. X2) THEN
          M = (Y2 - Y1)/(X2 - X1)
         B = (Y1*X2 - Y2*X1)/(X2 - X1)
      ENDIF
C
      DO J = 1,NROW
         Y = REAL(J)
         DO I = 1,NCOL
            X = REAL(I)
            IF (X1 .EQ. X2) THEN
               IF (Y .GE. Y1 .AND. Y .LE. Y2 .AND. X .EQ. X1) THEN
                  IMG(I,J) = IADD*IMG(I,J) + INT
               ENDIF
            ELSE
               IF(X.GE.X1 .AND. X.LE.X2) THEN
                 YPRIME = M*X + B
                 IF(ABS(Y-YPRIME).LE.1.0) THEN
                   IMG(I,J) = IADD*IMG(I,J) + INT
                 ENDIF
               ENDIF
            ENDIF
         ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *******************************************************************
C  *                                                                 *
C  *               *****  SUBROUTINE R4_IMG_TST_REC  *****              *
C  *                                                                 *
C  *    ADDS OR OVERWRITES FILLED/UNFILLED RECTANGLE TO MODEL IMAGE  *
C  *                                                                 *
C  *    AUGUST 1986 PURDUE UNIVERSITY VAX 11/750, L. MATTERN         *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).      *
C  *******************************************************************
C
      SUBROUTINE R4_IMG_TST_REC (IMG, NCOL, NROW, XCEN, YCEN, WIDTH,
     $     HEIGHT, IADD, INTVAR, FILL, IMG_MIN, IMG_MAX, IMG_AVG,
     $     IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, X, XCEN, Y, YCEN, WIDTH, HEIGHT
      REAL*4    X1, X2, Y1, Y2, XX1, XX2, YY1, YY2
      INTEGER*4 NCOL, NROW, IADD, INTVAR, I, J, FILL
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      LOGICAL   X_IN, Y_IN
C  ----------------------------------------------------------------------------
C
      IF(IADD.EQ.1) THEN
         IADD = 0
       ELSE
         IADD = 1
      ENDIF
C
      X1  = XCEN - WIDTH/2.0
      X2  = XCEN + WIDTH/2.0
      Y1  = YCEN - HEIGHT/2.0
      Y2  = YCEN + HEIGHT/2.0
      XX1 = REAL(INT(X1))
      XX2 = REAL(INT(X2))
      YY1 = REAL(INT(Y1))
      YY2 = REAL(INT(Y2))
      IF (XX1 .EQ. 0.0) XX1 = 1.0
      IF (YY1 .EQ. 0.0) YY1 = 1.0
C      
      DO J = 1,NROW
         Y = REAL(J)
         DO I = 1,NCOL
            X_IN = .FALSE.
            Y_IN = .FALSE.
            X = REAL(I)
            IF (FILL .EQ. 1) THEN
               IF ((X.GE.X1.AND.X.LE.X2).AND.(Y.GE.Y1.AND.Y.LE.
     1        Y2)) THEN
                  IMG(I,J) = IADD*IMG(I,J) + INTVAR
               ENDIF
            ELSE
               IF(X.GE.XX1 .AND. X.LE.XX2) X_IN = .TRUE.
               IF(Y.GE.YY1 .AND. Y.LE.YY2) Y_IN = .TRUE.
               IF((X_IN .AND.Y.EQ.YY2) .OR. (X_IN.AND.Y.EQ.YY1)
     1        .OR. (Y_IN .AND. X .EQ. XX1) .OR. (Y_IN .AND. X .EQ.
     2        XX2)) THEN
                  IMG(I,J) = IADD*IMG(I,J) + INTVAR
               ENDIF
            ENDIF
         ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C  *********************************************************************
C  *                                                                   *
C  *                *****  SUBROUTINE R4_IMG_TST_ARB  *****               *
C  *                                                                   *
C  *    ADDS OR OVERWRITES AN UNFILLED ARBITRARY FIGURE TO MODEL IMAGE *
C  *                                                                   *
C  *    NOVEMBER 17, 1986 PURDUE UNIVERSITY VAX 11/750, BY L. MATTERN  *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).        *
C  *********************************************************************
C
      SUBROUTINE R4_IMG_TST_ARB (IMG, NCOL, NROW, NUM, X, Y, IADD,
     1 INTVAR, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD, TMP, XDIF, YLO, YHI
      REAL*4    X(25), Y(25), XX, YY, M, B, YPRIME, XHI, XLO
      INTEGER*4 NCOL, NROW, I, J, INTVAR, IADD, N, NUM
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
C ---------------------------------------------------------------------
C
      IF(IADD.EQ.1) THEN
         IADD = 0
       ELSE
         IADD = 1
      ENDIF
C
      DO N = 1,NUM
         IF(N.LT.NUM) THEN
            YLO  = Y(N)
            YHI  = Y(N+1)
            XLO  = X(N)
            XHI  = X(N+1)
            XDIF = X(N+1) - X(N)
         ELSE
            YLO  = Y(NUM)
            YHI  = Y(1)
            XLO  = X(NUM)
            XHI  = X(1)
            XDIF = X(1) - X(NUM)
         ENDIF
C
         IF (XLO .GT. XHI) THEN
            TMP = XLO
            XLO = XHI
            XHI = TMP
         ENDIF
         IF (YLO .GT. YHI) THEN
            TMP = YLO
            YLO = YHI
            YHI = TMP
         ENDIF
C
         IF (N .LT. NUM .AND. XDIF .NE. 0.0) THEN
            M = (Y(N+1)-Y(N))/(X(N+1)-X(N))
            B = (Y(N)*X(N+1) - Y(N+1)*X(N))/(X(N+1) - X(N))
         ELSE IF (N .EQ. NUM .AND. XDIF .NE. 0.0) THEN
            M = (Y(1)-Y(NUM))/(X(1)-X(NUM))
            B = (Y(NUM)*X(1) - Y(1)*X(NUM))/(X(1) - X(NUM))
         ENDIF
C
         DO J = 1,NROW
            YY = REAL(J)
            DO I = 1,NCOL
               XX = REAL(I)
               IF (XDIF .EQ. 0.0) THEN      
                  IF (YY .GE. YLO .AND. YY .LE. YHI .AND. XX .EQ.
     1 X(N)) THEN
                     IMG(I,J) = IADD*IMG(I,J) + INTVAR
                  ENDIF
               ELSE
                  IF(XX.GE.XLO .AND. XX.LE.XHI) THEN
                    YPRIME = M * XX + B
                    IF(ABS(YY-YPRIME).LE.1.0) THEN
                      IMG(I,J) = IADD*IMG(I,J) + INTVAR
                    ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
      END

C ***********************************************************************
C *                                                      *
C *            ****** SUBROUTINE R4_IMG_WEIGHTFLT ******                  *
C *                                                      *
C *            Copied from routine IMG_AVGFLT, this routine expands on *
C *      the old one in that one can vary the weighting factor ('weight' *
C *      in parameter list), instead of always using weight=2, as was    *
C *      the case in IMG_AVGFLT. This is useful when one must have bet-  *
C *      ter control over data defocusing than is available with the     * 
C *      'ncycle' parameter. Example: gaussian noise, sent in 'img',     *
C *      will be slightly defocused to match the focus of raw micrograph *
C *      data.                                                * 
C *                                                      *
C *      CREATED: August, 1990  by nrd@bragg.bio.purdue.edu            *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).         *
C ***********************************************************************
C
      SUBROUTINE R4_IMG_WEIGHTFLT (IMG, NCOL, NROW, WEIGHT, NCYCLE, 
     1   IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      REAL*4    IMG_AVG, IMG_STD
      REAL*4    C, D, E, R4I2
      INTEGER*4 I, J, K
      INTEGER*4 NCOL, NROW, NCYCLE
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG_BKG, IMG(NCOL,NROW)
      real*4, allocatable :: img2(:,:)
      REAL*4        WEIGHT
      INTEGER   ALLOC_STAT
c  ----------------------------------------------------------------------------

      allocate(img2(ncol,nrow), stat=alloc_stat)
      if(alloc_stat > 0) then
         write(*,*) 'Unable to allocate array IMG2 in R4_IMG_AVERAGEFLT'
      endif

      DO K = 1,NCYCLE
C  ****************************
C  *    AVERAGE ALONG ROWS    *
C  ****************************
        DO J = 1,NROW
          C            = REAL(IMG(1,J))
          D            = REAL(IMG(2,J))
          E            = REAL(IMG(NCOL,J))
          IMG2(1,J)    = r4i2((E+WEIGHT*C+D)/(WEIGHT+2.0))
          IMG2(NCOL,J) = r4i2((C+WEIGHT*E+REAL(IMG(NCOL-1,J)))/ 
     1 (WEIGHT+2.0))
          DO I = 2,NCOL-1
            E         = REAL(IMG(I+1,J))
            IMG2(I,J) = r4i2((C+WEIGHT*D+E)/(WEIGHT+2.0))
            C         = D
            D         = E
          ENDDO
        ENDDO
C  ***********************************
C  *    NOW AVERAGE ALONG COLUMNS    *
C  ***********************************
        DO I = 1,NCOL
          C           = REAL(IMG2(I,1))
          D           = REAL(IMG2(I,2))
          E           = REAL(IMG2(I,NROW))
          IMG(I,1)    = r4i2((E+WEIGHT*C+D)/(WEIGHT+2.0))
          IMG(I,NROW) = r4i2((C+WEIGHT*E+REAL(IMG2(I,NROW-1)))/
     1 (WEIGHT+2.0))
          DO J = 2,NROW-1
            E        = REAL(IMG2(I,J+1))
            IMG(I,J) = r4i2((C+WEIGHT*D+E)/(WEIGHT+2.0))
            C        = D
            D        = E
          ENDDO
        ENDDO
      ENDDO
C
      CALL R4_IMG_STATS (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
      deallocate(img2)

      RETURN
      END

C  ****************************************************************
C  *                                                              *
C  *           *****     SUBROUTINE R4_IMG_X_IMG     *****           *
C  *                                                              *
C  *    COMPUTE IMG1*IMG2 AND STORE ANSWER IN IMG2                *
C  *                                                              *
C  *    WARNING: BEWARE OF POSSIBLE OVERFLOWS IN "IMAGE" AND      *
C  *     POSSIBLE NEED FOR RESCALING "IMAGE"                      *
C  *                                                              *
C  *    FEBRUARY 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER     *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.      *
C  *    Last modified  4-Dec-2001  By T. Baker (Rid of EM.CMM).   *
C  ****************************************************************
C
      SUBROUTINE R4_IMG_X_IMG (IMG1, IMG2, NCOL, NROW, IMG_MIN, 
     1   IMG_MAX, IMG_AVG, IMG_STD, IMG_BKG)
C
      IMPLICIT NONE
C
      include '../include/vax_minmax.inc'
C
      REAL *4   IMG_AVG, IMG_STD
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, ITMP, NUMCHARI4, N1
      INTEGER*4 I, ICOUNT
      real*4    IMG_MIN, IMG_MAX
      real*4    IMG1(1), IMG2(1), IMG_BKG
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      ICOUNT     = 0
      WRITE(*,1000)
C
      DO I = 1,ARRAY_SIZE
        ITMP = IMG1(I)*IMG2(I)
        IF(IABS(ITMP).GT.I2_MAX) THEN
           ICOUNT  = ICOUNT + 1
           IMG2(I) = ISIGN(1,ITMP)*I2_MAX
         ELSE
           IMG2(I) = ITMP
        ENDIF
      ENDDO
      IF(ICOUNT.NE.0) THEN
        PRINT *, '************************************'
        PRINT *, '* CAUTION: Overflow of INT*2 value *'
        PRINT *, '************************************'
        N1 = NUMCHARI4(ICOUNT)
        WRITE(*,'(A,I8,A)') ' There were ', ICOUNT, 
     1  ' over/under-flows'
      ENDIF
C
      CALL R4_IMG_STATS (IMG2, NCOL, NROW, IMG_MIN, IMG_MAX, IMG_AVG,
     1   IMG_STD, IMG_BKG)
C
      RETURN
C
 1000      FORMAT(' Compute IMG1*IMG2 and store answer in IMG2')
      END

C  **************************************************************
C  *                                                *
C  *              ***** SUBROUTINE R4_IMG_X_SUM *****            *
C  *                                                *
C  *            Computes the sum of the multiplications of      *
C  *      corresponding pixels in two same-size real*4 images.    *
C  *                                                *
C  *      CREATED: June 6, 1991  by nrd@bragg.bio.purdue.edu      *
C  **************************************************************

      SUBROUTINE R4_IMG_X_SUM (IMG1, IMG2, NCOL, NROW, BKG1, BKG2, SUM)
C
      IMPLICIT NONE
C
      REAL*4    SUM
      real*4    IMG1(1), IMG2(1)
      INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, BKG1, BKG2
C  ----------------------------------------------------------------------------
C
      ARRAY_SIZE = NCOL*NROW
      SUM=0.0
C
      DO I = 1,ARRAY_SIZE
        SUM = SUM + REAL(IMG1(I)-BKG1)*REAL(IMG2(I)-BKG2)
      ENDDO
C
      RETURN
      END
